@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G.pm,v 1.4 2002/07/30 17:40:56 gaou Exp $
@@ -37,6 +37,12 @@ use base qw(G::IO Exporter);
 use SubOpt;
 use Rcmd;
 
+# import Rcmd classes
+use Rcmd::Clustering;
+use Rcmd::Normality;
+use Rcmd::Multivariate;
+use Rcmd::Summary;
+
 # import messenger
 use G::Messenger;
 
@@ -65,15 +71,12 @@ use G::Seq::Util;
 use G::Seq::Align;
 use G::Seq::GenomeMap;
 use G::Tools::Graph;
-use G::Tools::EMBOSS;
-use G::Tools::Alignment;
 use G::Tools::COGs;
 use G::Tools::GPAC;
 use G::Tools::GOA;
 use G::Tools::Statistics;
 use G::Tools::GMap;
 use G::Tools::WebServices;
-use G::Tools::DotE;
 
 our @EXPORT = qw(load opt_list method_list);
 
@@ -91,22 +94,22 @@ push(@EXPORT, @G::Seq::Util::EXPORT);
 push(@EXPORT, @G::Seq::Align::EXPORT);
 push(@EXPORT, @G::Seq::GenomeMap::EXPORT);
 push(@EXPORT, @G::Tools::Graph::EXPORT);
-push(@EXPORT, @G::Tools::EMBOSS::EXPORT);
-push(@EXPORT, @G::Tools::Alignment::EXPORT);
 push(@EXPORT, @G::Tools::COGs::EXPORT);
 push(@EXPORT, @G::Tools::GPAC::EXPORT);
 push(@EXPORT, @G::Tools::GOA::EXPORT);
 push(@EXPORT, @G::Tools::Statistics::EXPORT);
 push(@EXPORT, @G::Tools::GMap::EXPORT);
 push(@EXPORT, @G::Tools::WebServices::EXPORT);
-push(@EXPORT, @G::Tools::DotE::EXPORT);
-
 push(@EXPORT, @G::Shell::EUtils::EXPORT);
 push(@EXPORT, @G::Shell::Help::EXPORT);
 push(@EXPORT, @G::DB::SDB::EXPORT);
 push(@EXPORT, @G::DB::BDB::EXPORT);
-push(@EXPORT, @SubOpt::EXPORT);
 push(@EXPORT, @G::Messenger::EXPORT);
+push(@EXPORT, @SubOpt::EXPORT);
+push(@EXPORT, @Rcmd::Clustering::EXPORT);
+push(@EXPORT, @Rcmd::Multivariate::EXPORT);
+push(@EXPORT, @Rcmd::Summary::EXPORT);
+push(@EXPORT, @Rcmd::Normality::EXPORT);
 
 # export plugin functions (overrides all)
 push(@EXPORT, @G::DynamicLoader::EXPORT);
@@ -153,7 +156,7 @@ sub method_list{
 	    next_cds previous_feature previous_cds startcodon stopcodon before_startcodon
 	    after_startcodon before_stopcodon after_stopcodon around_startcodon around_stopcodon
 	    get_exon get_exons get_intron pos2feature pos2gene gene2id next_locus clone del_key
-	    reverse_strand relocate_origin disclose bioperl set_essentiality
+	    reverse_strand relocate_origin disclose set_essentiality
 	    /;
     }else{
 	my @return;
@@ -291,6 +294,9 @@ G - G-language Genome Analysis Environment core interface module
          First option is the filename of the database. Default format is
          the GenBank database. Database format is guessed from the extensions.
          (eg. .gbk => GenBank, .fasta => FASTA, .embl => EMBL)
+         Most of the major sequence formats are supported, including
+         Fasta, Fastq, GenBank, EMBL, Swiss-Prot, GCG, PIR, and so on.
+
          Flatfile can be gzipped. If the file extension ends with ".gz",
          load() can automatically handle it as compressed file.
 
@@ -307,12 +313,20 @@ G - G-language Genome Analysis Environment core interface module
          Uniform Sequence Address (USA) keys.
          http://emboss.sourceforge.net/docs/themes/UniformSequenceAddress.html
          Currently supported database keys are: 
-            swiss, genbank, genpept, embl, refseq 
+            swiss, genbank, embl, refseq 
          eg. 
             $gb = load("embl:xlrhodop");
             $gb = load("genbank:AY063336")
             $gb = load("swiss:ROA1_HUMAN")
 
+         For bacteria, name of the species, its abbreviations, and NCBI 
+         taxonomy IDs can also be used.
+         All of the following examples load E.coli K12 genome (NC_000913).
+         eg. 
+            $gb = load("Escherichia coli");
+            $gb = load("e.coli k12");
+            $gb = load("511145");
+
          Subsequent arguments work as options. Multiple options can be given 
          in any order.
 
@@ -329,11 +343,11 @@ G - G-language Genome Analysis Environment core interface module
            'multiple locus'          this option merges multiple loci in the 
                                      database and load the information
                                      as G-language instance.
-
-           'bioperl'                 this option creates a G instance from 
-                                     a bioperl object. 
-                                     eg. $bp = $bp->next_seq();       # bioperl
-                                         $gb = load($bp, "bioperl"); # G
+                                     
+                                     By supplying a number with this option, 
+                                     eg. 'multiple locus 50'
+                                     merged sequence contains 50 'N's between
+                                     the contigs. By default, no 'N' is inserted.
 
            'longest ORF annotation'  this option predicts genes with longest ORF
                                      algorithm (longest frame from start codon
@@ -344,19 +358,6 @@ G - G-language Genome Analysis Environment core interface module
            'Fastq'              this option loads a FastQ format database.
            'EMBL'               this option loads a EMBL  format database.
 
-               - following options require bioperl installation -
-
-           'swiss'              this option loads a swiss format database.
-           'SCF'                this option loads a SCF   format database.
-           'PIR'                this option loads a PIR   format database.
-           'GCG'                this option loads a GCG   format database.
-           'raw'                this option loads a raw   format database.
-           'ace'                this option loads a ace   format database.
-           'net GenBank'        this option loads a GenBank format database from 
-                                NCBI database. With this option, the first value to 
-                                pass to load() function will be the accession 
-                                number of the database.
-
 =cut
 
 
