@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -31,7 +31,6 @@ package G::IO;
 
 use strict;
 use base qw(G::IO::Handler G::IO::GenBankI G::IO::GenBankO G::DB::Handler);
-use autouse 'Bio::Perl'=>qw(read_sequence write_sequence get_sequence);
 
 use G::Messenger;
 use G::DB::SDB;
@@ -47,7 +46,7 @@ use File::Basename;
 use File::ShareDir ':ALL';
 use autouse 'File::Temp'=>qw(tempfile);
 
-our $VERSION = '1.8.13';
+our $VERSION = '1.9.0';
 
 #::::::::::::::::::::::::::::::
 #          Perldoc
@@ -78,7 +77,6 @@ Kazuharu Arakawa, gaou@sfc.keio.ac.jp
 #          Variables
 #::::::::::::::::::::::::::::::
 
-my $outfile = '';
 my $loaded = 0;
 my @instances;
 
@@ -100,6 +98,7 @@ sub new {
 	   lc($filename) eq 'mgen'  || 
 	   lc($filename) eq 'cyano' ||
 	   lc($filename) eq 'bbur'  ||
+	   lc($filename) eq 'lambda'  ||
 	   lc($filename) eq 'plasmidf'
 	   ){
 	$filename = dist_file('g-language', "genomes/$filename.gbk");
@@ -115,8 +114,7 @@ sub new {
 
     my $no_msg                 = 0;
     my $multiple_locus         = 0;
-    my $bioperl                = 0;
-    my $netgbk                 = 0;
+    my $multiple_locus_len     = 0;
     my $longest_ORF_annotation = 0;
     my $glimmer_annotation     = 0;
     my $no_cache               = 0;
@@ -130,16 +128,17 @@ sub new {
     $gzip = $filename =~ s/\.gz$//;
     
     foreach my $opt_tmp (@options){
+	next if(length($opt_tmp) < 1);
+
 	my $opt = lc($opt_tmp);
 
 	if ($opt =~ /no msg/){
 	    $no_msg = 1;
-	}elsif ($opt =~ /multiple locus/){
+	}elsif ($opt =~ /multiple locus\D*(\d*)/){
 	    $multiple_locus = 1;
+	    $multiple_locus_len = $1;
+#	    msg_error("Multiple locus len: $multiple_locus_len");
 	    $no_cache = 1;
-	}elsif ($opt =~ /bioperl/){
-	    $bioperl = 1;
-	    $format = 'bioperl';
 	}elsif ($opt =~ /longest orf/){
 	    $longest_ORF_annotation = 1;
 	}elsif ($opt =~ /glimmer annotation/){
@@ -149,9 +148,6 @@ sub new {
 	}elsif ($opt =~ /force cache/){
 	    $force_cache = 1;
 	    $no_cache = 1;
-	}elsif ($opt =~ /net GenBank/){
-	    $netgbk = 1; 
-	    $format = 'net genbank';
 	}elsif (   $opt eq 'fasta' || $opt eq 'embl' 
 		|| $opt eq 'swiss' || $opt eq 'scf' 
 		|| $opt eq 'pir'   || $opt eq 'gcg' 
@@ -161,28 +157,21 @@ sub new {
 		|| $opt eq 'fastq' || $opt eq 'genbank' ){
 	    $format = $opt;
 	}else{
-	    unless (lc($opt) eq 'net genbank'){
-		warn("Warning: Unknown Option $opt at \"new G\"\n");
-	    }
+#	    warn("Warning: Unknown Option $opt at \"new G\"\n");
 	}
     }
-
+    
     $this->loaded_msg() unless ($no_msg);
 
     $format = G::IO::Handler::_interpret_format($filename) unless(length $format);
 
-    if($format eq 'RefSeq'){
+    if($format =~ /^named:(.*)$/){
+	$format = 'usa';
+	$filename = "genbank:$1";
+    }elsif($format eq 'RefSeq'){
 	$format = 'usa';
 	$filename = "genbank:$filename";
-    }elsif($format eq 'net GenBank'){
-	$format = 'usa';
-	$filename = "genbank:$filename";
-    }elsif($format eq 'net GenPept'){
-	$format = 'usa';
-	$filename = "genbank:$filename";
-    }
-    
-    if($format eq 'fastq' || $format eq 'fasta'){
+    }elsif($format eq 'fastq' || $format eq 'fasta'){
 	$no_cache = 1 unless($force_cache);
     }
 
@@ -195,10 +184,6 @@ sub new {
 	$this->seq_info() unless ($no_msg);
 	my $now = time();
 	utime $now, $now, $cachedFile;
-    }elsif ($bioperl || $format eq 'bioperl'){
-        $this = _bp2gb($filename);
-	$this->seq_info() unless ($no_msg);
-
     }elsif ($format eq 'usa'){
 	my ($dbname, $entryname) = split(/:/, $filename, 2);
 
@@ -206,13 +191,13 @@ sub new {
 	    $dbname = 'uniprot';
 	}
 
-	if($dbname eq 'refseq'){
-	    $dbname = 'genbank';
+	if($dbname eq 'refseq' || $dbname eq 'genbank'){
+	    $dbname = 'nuccore';
 	}
 
 	msg_error("Retrieving sequence $entryname from " . uc($dbname) . "...\n");
 
-	if($dbname eq 'genbank' || $dbname eq 'embl'){
+	if($dbname eq 'nuccore' || $dbname eq 'embl'){
 	    my $path = _sdb_path() . $entryname . '.' . $dbname;
 	    mirror('http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname, $path);
 	    if($no_msg){
@@ -231,16 +216,7 @@ sub new {
 	    }
 	    croak('Sequence not found in database at http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname) unless(length($this->{SEQ}));
 	}else{
-	    my $bp;
-	    
-	    eval {
-		$bp = get_sequence($dbname, $entryname);
-	    };
-	    die("$@ Could not retrieve $filename\n") if $@;
-	    
-	    $this = _bp2gb($bp);
-	    $this->seq_info() unless($no_msg);
-	    $this->{BIOPERL} =  $bp;
+	    die("Unsupported USA: $filename\n");
 	}
     }else{
 
@@ -252,9 +228,20 @@ sub new {
 	    $this = new G::IO::FastQI;
 	}elsif ($format ne 'genbank'){
 
-	    (undef, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
-	    my $in = read_sequence($filename);
-	    my $out = write_sequence(">$outfile", "genbank", $in);
+	    my($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+
+	    require LWP::UserAgent;
+	    my $ua = LWP::UserAgent->new;
+	    my $response = $ua->post("http://rest.g-language.org/emboss/",
+				     Content_Type=>'multipart/form-data',
+				     Content=>[file1 => [$filename], 'arg'=>'seqret/osformat2=genbank/-feature']
+				     );
+
+	    if ($response->is_success) {
+		print $fh $response->decoded_content;
+	    } else {
+		die $response->status_line;
+	    }
 
 	    $filename = $outfile;
 	    $locus_msg = 'no msg';
@@ -267,7 +254,7 @@ sub new {
 	    my $tmp = $this->clone();
 	    undef(%$this);
 
-	    multi_locus($this, $tmp);
+	    multi_locus($this, $tmp, $multiple_locus_len);
 	    
 	    $this->set_gene_aliases();
 	    $this->seq_info() unless($no_msg);
@@ -328,7 +315,7 @@ sub loaded_msg {
 	         Arakawa K. et al. (2008) Genes, Genomes and Genomics.
 
  	      License: GNU General Public License
-	      Copyright (C) 2001-2011 G-language Project
+	      Copyright (C) 2001-2014 G-language Project
 	      Institute for Advanced Biosciences, Keio University, JAPAN 
 
 	     __/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/
@@ -338,25 +325,17 @@ sub loaded_msg {
 }
 
 
-sub _bp2gb {
-    my $bpobj = shift;
-
-    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
-    write_sequence('>' . $outfile, "genbank", $bpobj);
-    my $this = new G::IO($outfile, "no msg");
-
-    return $this;
-}
-
-
 sub multi_locus{
     my $this = shift;
-    my $gb = shift;
+    my $gb  = shift;
+    my $len = shift || 0;
     my $lng;
     my $i = 0;
     my $f = 1;
     my $c = 1;
     
+#    msg_error("Multiple locus Length Sub: $len");
+
     do{
         my $F = 1;
         $lng = length($this->{"SEQ"});
@@ -396,7 +375,7 @@ sub multi_locus{
             $f++;
             $F++;
         }
-        $this->{"SEQ"} .= $gb->{"SEQ"};
+        $this->{"SEQ"} .= $gb->{"SEQ"} . 'n' x $len;
         $i++;
 
     }while($gb->next_locus("no msg"));
