@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GenBankI.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -53,9 +53,10 @@ sub new{
 }
 
 sub DESTROY {
-    return unless(defined $FH{$_[0]->{INSTANCE}});
-    close ($FH{$_[0]->{INSTANCE}});
-    delete($FH{$_[0]->{INSTANCE}});
+    if(exists $_[0]->{INSTANCE} && $FH{$_[0]->{INSTANCE}}){
+	close ($FH{$_[0]->{INSTANCE}});
+	delete($FH{$_[0]->{INSTANCE}});
+    }
 }
 
 sub open_gb {
@@ -94,7 +95,10 @@ sub next_locus{
     if (defined($this->{SEQ})){
         $this->seq_info() unless($msg);
         return 1;
-    }else{
+    }elsif(defined($this->{LOCUS}->{id})){
+	carp('No sequence found for ' . $this->{LOCUS}->{id});
+	return -1;
+    }else{	
         return 0;
     }
 }
