@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GenBankI.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -78,7 +78,7 @@ sub next_locus{
 
     $this->read_locus($msg);
     $this->read_header();
-    $this->read_features();
+    $this->read_features() unless($this->{G_INTERNAL_MSG} eq 'skip');
     $this->getnucs();
 
     if (length($this->{SEQ}) > 0){
@@ -136,10 +136,14 @@ sub read_header{
     my $fh = $FH{$this->{INSTANCE}};
     while($line = <$fh>){
 	last if ($line =~ /^FEATURES/);
+	if($line =~ /^ORIGIN/){
+	    $this->{G_INTERNAL_MSG} = 'skip';
+	    last;
+	}
 	$this->{HEADER} .= $line;
     }
 
-    $this->{HEADER} =~ /((Archaea|Bacteria|Eukaryota);.*?)\n\S/s;
+    $this->{HEADER} =~ /((Archaea|Bacteria|Eukaryota|Viruses|other sequences);*.*?)\n\S/s;
     $this->{TAXONOMY}->{0} = $1;
     $this->{TAXONOMY}->{0} =~ s/\n//g;
     $this->{TAXONOMY}->{0} =~ s/\s+ / /g;
@@ -174,6 +178,7 @@ sub read_header{
     $this->{$_} =~ s/\n//g  foreach(qw(DEFINITION ACCESSION SOURCE ORGANISM));
 
     my @organism = split(/ /, $this->{ORGANISM});
+    shift @organism if($organism[0] =~ /Candidatus/); 
     $this->{TAXONOMY}->{genus}   = shift @organism;
     $this->{TAXONOMY}->{species} = shift @organism;
 }
@@ -333,7 +338,13 @@ sub read_features{
 		}
 		$tmp =~ s/\s+/ /g;
 		$tmp =~ s/ //g if ($tag eq 'translation');
-		$this->{"FEATURE$num"}->{$tag} = $tmp;
+
+		if(length($this->{"FEATURE$num"}->{"$tag"})){
+		    $this->{"FEATURE$num"}->{"$tag"} .= "\t" . $tmp;
+		}else{
+		    $this->{"FEATURE$num"}->{"$tag"} = $tmp;
+		}
+
 	    }elsif (/\/(\w+)=(\d+)/){
 		$this->{"FEATURE$num"}->{$1} = $2;
 	    }elsif (/\/(\w+)=\((.*)\)/){
