@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO::Handler.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -39,7 +39,7 @@ use G::DB::SDB;
 use File::ShareDir ':ALL';
 
 use autouse 'File::Temp'=>qw(tempfile);
-use autouse 'Bio::Perl'=>qw(read_sequence write_sequence get_sequence);
+use autouse 'LWP::Simple'=>qw(mirror);
 
 #::::::::::::::::::::::::::::::
 #          Perldoc
@@ -141,6 +141,47 @@ sub clone {
 }
 
 
+=head2 $gb->annotate()
+
+   Name: $gb->annotate()   -   reannotate the genome using Restauro-G
+
+   Description:
+      This method reannotates the entire genome, if coding sequences are already available.
+      Added features includes:
+      rs_des: description of protein
+      rs_xr:  database cross references, including EMBL, RefSeq, KEGG, BioCyc, BRENDA, GO, InterPro, Pfam
+      rs_com: comments
+  Usage: 
+      $new_gb = $gb->annotate();
+
+   REST: 
+      http://rest.g-language.org/NC_000913/annotate
+
+   References:
+      1. Tamaki S, Arakawa K, Kono N, Tomita M (2007) "Restauro-G: A Rapid Genome Re-Annotation 
+         System for Comparative Genomics", Genomics Proteomics Bioinformatics, 5(1): 53-58.
+
+   Author:
+      Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+   History:
+      20120830-01 initial posting
+
+=cut
+
+sub annotate {
+    my $this = shift;
+
+    my($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+    $this->make_gb($outfile);
+
+    require LWP::UserAgent;
+    my $ua = LWP::UserAgent->new;
+    my $id = $ua->post('http://rest.g-language.org/upload/upl.pl', 'Content_Type'=>'form-data', 'Content'=>['file'=>[$outfile]])->content;
+    mirror("http://rest.g-language.org/$id/restauro", $outfile);
+    my $new = new G::IO($outfile, "no cache", "no msg");
+    return $new;
+}
 
 =head2 $gb->disclose()
 
@@ -276,7 +317,7 @@ sub find {
          Given the start and end positions (starting from 0 as in Perl),
          returns the sequence specified.
 
-         eg. $gb->getseq(1,3); # returns the 2nd, 3rd, and 4th nucleotides.
+         eg. $gb->getseq(2,4); # returns the 3rd, 4th, and 5th nucleotides.
 
    Options:
        -circular   when the first position is larger than the second position,
@@ -303,11 +344,10 @@ sub getseq {
 	return substr($this->{SEQ}, $start, $end-$start+1);
     }else{
 	if($option =~ /circ/){
-	    return substr($this->{SEQ}, $start) . 
-		substr($this->{SEQ}, 0, $end + 1);
+	    return substr($this->{SEQ}, $start) . substr($this->{SEQ}, 0, $end + 1);
 	}else{
 	    my ($start2, $end2) = sort {$a <=> $b} ($start, $end);
-	    return substr($this->{SEQ}, $start, $end-$start+1);
+	    return substr($this->{SEQ}, $start2, $end2-$start2+1);
 	}
     }
 }
@@ -1001,6 +1041,7 @@ sub get_exons {
 sub seq_info {
     my $this = shift;
     my $nomsg = shift;
+    my $type = shift;
 
     my $length = length($this->{SEQ});
 
@@ -1014,13 +1055,15 @@ sub seq_info {
 
     unless($nomsg){
 	$msg .= sprintf "\n\nAccession Number: %s\n", $this->{LOCUS}->{id} if(length($this->{LOCUS}->{id}));
+	$msg .= sprintf "\nDefinition: %s\n", $this->{DEFINITION} if(length($this->{DEFINITION}));
+	$msg .= sprintf "\n  Merged LOCUS Count : %9d\n" , scalar(grep{/LOCUS/} keys(%$this)) - 2 if(exists($this->{LOCUS2}));
 	$msg .= sprintf "\n  Length of Sequence : %9d\n" , $length;
 	
 	if($length == 0){
 	    $msg .= sprintf "\n  No Sequence Found.\n\n";
 	}elsif($symbol > 5){
 	    $msg .= sprintf "\n  Sequence seems to be invalid (neither nucleotide nor amino acid).\n\n";
-	}elsif($others > $a + $t + $g + $c){
+	}elsif($others > $a + $t + $g + $c && $type != 1){
 	    amino_info($this);
 	    return;
 	}else{
@@ -1031,6 +1074,11 @@ sub seq_info {
 	    $msg .= sprintf "              Others : %9d (%.2f\%)\n" , $others,  $others / $length * 100;
 	    $msg .= sprintf "          AT Content :    %.2f\%\n"    , ($a + $t) / $length * 100;
 	    $msg .= sprintf "          GC Content :    %.2f\%\n\n"  , ($g + $c) / $length * 100;
+
+	    my ($genes, $cds, $trna, $rrna) = (scalar($this->gene()), scalar($this->cds()), scalar($this->tRNA()), scalar($this->rRNA()));
+	    if($cds > 0){
+		$msg .= sprintf "      Number of genes :    %d (CDSs: %s, tRNAs: %s, rRNAs: %s) \n"    , $genes, $cds, $trna, $rrna;
+	    }
 	}
 	&msg_send($msg);
     }
@@ -1047,7 +1095,7 @@ sub seq_info {
          to 0 when $genome->{$feature}->{pseudo} is true.
 
    REST: 
-      http://rest.g-language.org/NC_000913/disable_pseudogenes()
+      http://rest.g-language.org/NC_000913/disable_pseudogenes
 
 =cut
 
@@ -1270,6 +1318,9 @@ sub next_feature{
     my $feature = shift || 'FEATURE0';
     my $opt = shift;
 
+    my @features = $this->feature($opt);
+    return undef if ($feature eq $features[-1]);
+
     $feature = $this->{$feature}->{left} if ($feature =~ /^INTER/);
 
     my $i = $this->{$feature}->{feature};
@@ -1312,9 +1363,10 @@ sub previous_feature{
     my $feature = shift || 'FEATURE0';
     my $opt = shift;
 
+    return undef if ($feature eq 'FEATURE0');
     $feature = $this->{$feature}->{right} if ($feature =~ /^INTER/);
 
-    my $i = $this->{$feature}->{feature} if ($feature =~ /^CDS/);
+    my $i = $this->{$feature}->{feature};
     $i --;
 
     while(%{$this->{"FEATURE$i"}}){
@@ -1387,13 +1439,13 @@ sub intergenic {
 
     set_intergenic($this) unless(defined($this->{INTER1}));
 
-    while(%{$this->{"INTER" . ($i + 1)}}){
+    while($this->{"INTER" . ($i + 1)}){
 	$i ++;
 
 	if ($opt ne 'all'){
 	    next if ($this->{"INTER$i"}->{on} == 0);
 	}
-
+	
 	push (@cds, "INTER$i");
     }
 
@@ -1711,35 +1763,31 @@ sub _interpret_format {
     my $filename = shift;
     my $ref = ref $filename;
 
-    if ($ref =~ /Bio::Seq/){
-	return 'bioperl';
-    }elsif ($filename =~ /^(.*?):(.*)/){
+    if ($filename =~ /^(.*?):(.*)/){
 	unless(lc($1) =~ /(swall|swiss|sw|uniprot|genbank|genpept|embl|refseq)/){
 	    croak("Unsupported database name for USA. Supported databases are\n" .
 		"swiss, sw, uniprot, genbank, genpept, embl, or refseq\n");
 	}
 	return "usa";
-    }elsif (lc($filename) =~ /\.(gb|gbk|gbank|genbank)$/i){
+    }elsif (lc($filename) =~ /\.(gb|gbk|gbank|genbank|nuccore)$/i){
 	return 'genbank';
     }elsif (lc($filename) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
 	return 'fasta';
     }elsif (lc($filename) =~ /\.(fastq|fq)/i){
 	return 'fastq';
-    }elsif ($filename =~ /^NC_\d+$/i){
+    }elsif (lc($filename) =~ /\.(embl)/i){
+	return 'embl';
+    }elsif ($filename =~ /^NC_\d+$/i ||
+	    $filename =~ /^NP_\d+$/i ||
+	    $filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+$/i ||
+	    $filename =~ /^NC_\d+\.\d+$/i ||
+	    $filename =~ /^NP_\d+\.\d+$/i ||
+	    $filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+\.\d+$/i
+	    ){
 	return 'RefSeq';
-    }elsif ($filename =~ /^NP_\d+$/i){
-	return 'net GenPept';
-    }elsif ($filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+$/i){
-	return 'net GenBank';
-    }elsif ($filename =~ /^NC_\d+\.\d+$/i){
-	return 'RefSeq';
-    }elsif ($filename =~ /^NP_\d+\.\d+$/i){
-	return 'net GenPept';
-    }elsif ($filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+\.\d+$/i){
-	return 'net GenBank';
-    }else{
-	require Bio::SeqIO;
-	my $format = Bio::SeqIO->_guess_format($filename);
+    }elsif(-e $filename){
+	my @tmp = split(/\./, $filename);
+	my $format = $filename =~ /\./ ? pop(@tmp) : '';
 
 	if (length $format){
 	    return $format;
@@ -1760,7 +1808,11 @@ sub _interpret_format {
 	    }
 	    close(FILE);
 	}
+    }else{
+	my $id = readFile("http://rest.g-language.org/gbp/orgname.cgi?id=$filename", 1);
+	return ("named:$id") if(length($id));
     }
+
 }
 
 =head2 $gb->output()
@@ -1774,6 +1826,7 @@ sub _interpret_format {
 
          eg. $gb->output("my_genome.embl", "EMBL");
              $gb->output("my_genome.gbk"); # with GenBank you can ommit the option.
+             $gb->output("my_genome.gff3");
 
    REST: 
       http://rest.g-language.org/NC_000913/output/embl (convert input genome to EMBL format)
@@ -1793,46 +1846,28 @@ sub output {
     }elsif(length $option){
 	my $in;
 
-	if(length $gb->{BIOPERL}){
-	    $in = $gb->{BIOPERL};
-	}else{
-	    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
-	    $gb->make_gb($outfile);
-	    $in = read_sequence($outfile);
-	    unlink($outfile);
+	my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+	$gb->make_gb($outfile);
+
+	require LWP::UserAgent;
+	my $ua = LWP::UserAgent->new;
+            my $response = $ua->post("http://rest.g-language.org/emboss/",
+                                     Content_Type=>'multipart/form-data',
+                                     Content=>[file1 => [$outfile], 'arg'=>"seqret/osformat2=$option/-feature"]
+                                     );
+
+	if ($response->is_success) {
+	    writeFile($response->decoded_content, $file);
+	} else {
+	    die("Error in output:\n" . $response->status_line . "\n File format \"$option\"of \"$file\" is not supported.");
 	}
 
-	write_sequence(">$file", $option, $in);
+	unlink($outfile);
     }else{
 	&msg_error("G::output - Unknown format to output.");
     }
 }
 
-=head2 $gb->bioperl()
-
-   Name: $gb->bioperl()   -   output the G instance as Bio::SeqIO object
-
-   Description:
-         Outputs the G-language data object as Bio::SeqIO object.
-
-         eg. $bp = $gb->bioperl();
-
-   REST: 
-      http://rest.g-language.org/NC_000913/bioperl
-
-=cut
-
-sub bioperl {
-    require Bio::SeqIO;
-    my $this = shift;
-    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
-    $this->output($outfile);
-    
-    my $in = Bio::SeqIO->new(-format => "genbank", -file =>$outfile);
-    my $bp = $in->next_seq();
-
-    return $bp;
-}
 
 
 
