@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -182,7 +182,7 @@ sub readFile{
 	    }elsif(/^\/\//){
 		$key = '';
 	    }elsif(/^(..)\s+(.*)/){
-                $sprot->{$key}->{$1} .= $2;
+                $sprot->{$key}->{$1} .= length($sprot->{$key}->{$1}) ? ' ' . $2 : $2;
 	    }
 	}
 	close(FILE);
@@ -191,12 +191,12 @@ sub readFile{
     }
 
     unless($format){
-	if (lc($file) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
+	if ($file =~ /\:\/\//){
+	    $format = 'url';
+	}elsif (lc($file) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
 	    $format = 'fasta';
 	}elsif (lc($file) =~ /\.(fastq|fq)/i){
 	    $format = 'fastq';
-	}elsif ($file =~ /\:\/\//){
-	    $format = 'url';
 	}else{
 	    unless(-e $file){
 		$format = 'command';
@@ -219,6 +219,7 @@ sub readFile{
 	    $hash{$gb->{LOCUS}->{id}} = $gb->{SEQ};
 	}while($gb->next_locus('no msg'));
 
+	msg_error('Loaded ' . scalar(keys(%hash)) . ' ' . uc($format) . ' sequences.');
 	return %hash;
     }elsif($format eq 'command'){
 	foreach my $line (`$file`){
