@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Consensus.pm,v 1.2 2002/07/30 17:40:56 gaou Exp $
@@ -46,7 +46,7 @@ our @EXPORT = qw(
 	     consensus_z
 );
 
-#__DATA__
+__DATA__
 
 #::::::::::::::::::::::::::::::
 #     Perldoc
@@ -83,13 +83,14 @@ G::Seq::Consensus - Analysis methods related to sequence consensus
     internal usage.
 
   Options:
-     -position   either "start" (around start codon) or "end" (around stop codon)
-                 to create the PWM (default: "start");
-     -PatLen     length of oligomer to count (default: 3)
-     -upstream   length upstream of specified position to create PWM (default: 30)
-     -downstream length downstream of specified position to create PWM (default: 30)
-     -filename   outfile file name (default: consensus.csv)
-     -output     "stdout" for standard output, "f" for file
+    -output     "stdout" for standard output (default: "stdout")
+                "f" for file output in directory "data"
+    -filename   output filename (default: "base_counter.csv")
+    -position   either "start" (around start codon) or "end" (around stop codon)
+                to create the PWM (default: "start");
+    -PatLen     length of oligomer to count (default: 3)
+    -upstream   length upstream of specified position to create PWM (default: 30)
+    -downstream length downstream of specified position to create PWM (default: 30)
 
   Author: 
     Koya Mori(mory@g-language.org)
@@ -100,7 +101,7 @@ G::Seq::Consensus - Analysis methods related to sequence consensus
 =cut
 
 sub base_counter{
-    &opt_default(output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
+    &opt_default(output=>"stdout",filename=>"base_counter.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -189,7 +190,7 @@ sub base_counter{
 #need &bun.
 #&base_print(pointer Genome,  string position,  int pattern_length, int upstream_limit,  int downstream_limit,  boolean debug);
 sub _base_printer{
-    &opt_default(output=>"stdout",filename=>"consensus.csv");
+    &opt_default(output=>"stdout",filename=>"base_counter.csv");
     my @args=opt_get(@_);
 
     my $hash=shift @args;
@@ -205,7 +206,8 @@ sub _base_printer{
 
 
     if($printer eq "f"){
-	open(FILE,">>$filename");
+        mkdir ("data", 0777);
+        open(FILE,">>data/$filename");
 	print FILE "Pattern";
 	for(my $i=$upstream;$i>-$downstream;$i--){
 	    print FILE ",$i";
@@ -293,14 +295,18 @@ sub logmod{
   Usage:
     ($position, "entropy", $upstream-length, $downstream-length, @entropyvalues) = base_entropy($genome);
 
-  Options:
-     -position   either "start" (around start codon) or "end" (around stop codon)
-                 to create the PWM (default: "start");
-     -PatLen     length of oligomer to count (default: 3)
-     -upstream   length upstream of specified position to create PWM (default: 30)
-     -downstream length downstream of specified position to create PWM (default: 30)
-     -filename   outfile file name (default: consensus.csv)
-     -output     "g" for graph, "f" for file, "show" for display
+ Options:
+    -output     "stdout" for standard output
+                "f" for file output in directory "data",
+                "g" for graph output in directory "graph",
+                "show" for graph output and display (default: "show")
+    -filename   output filename (default: "base_entropy.png" for -output=>"g",
+                                          "base_entropy.csv" for -output=>"f")
+    -position   either "start" (around start codon) or "end" (around stop codon)
+                to create the PWM (default: "start");
+    -PatLen     length of oligomer to count (default: 3)
+    -upstream   length upstream of specified position to create PWM (default: 30)
+    -downstream length downstream of specified position to create PWM (default: 30)
 
   Author: 
     Koya Mori(mory@g-language.org)
@@ -311,7 +317,7 @@ sub logmod{
 =cut
 
 sub base_entropy{
-    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
+    &opt_default(output=>"show",filename=>"base_entropy.png",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -331,11 +337,13 @@ sub base_entropy{
     my $tmp;
     my @y;
 
-    $hash=base_counter($gb,-output=>"n",-filename=>$filename,-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
+    $hash=base_counter($gb,-output=>"n",-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
     $downstream++;
     
     if($printer=~/f/){
-	open(FILE,">>$filename");
+	$filename =~ s/\.png$/\.csv/;
+	mkdir ("data", 0777);
+	open(FILE,">>data/$filename");
     }
 
     for(my $i=$upstream;$i>-$downstream;$i--){
@@ -370,7 +378,7 @@ sub base_entropy{
 	if($printer=~/f/){
 	    printf FILE "%d,%.5f\n",-$i ,$entropy[$upstream-$i];
 	}
-	if($printer!~/[fn]/){
+	if($printer eq 'stdout'){
 	    &msg_send(sprintf("%d\t%.5f\n",-$i ,$entropy[$upstream-$i]));
 	}
 	push(@y,-$i);
@@ -378,10 +386,10 @@ sub base_entropy{
     close(FILE);
     
     if($printer=~/g/ || $printer=~/show/){
-	&G::Tools::Graph::_UniMultiGrapher(\@y,\@entropy,-filename=>"base_entropy.png",-x=>"position",-y=>"entropy",-title=>"entropy");
+	&G::Tools::Graph::_UniMultiGrapher(\@y,\@entropy,-filename=>$filename,-x=>"position",-y=>"entropy",-title=>"entropy");
     }
 
-    msg_gimv('graph/base_entropy.png') if($printer=~/show/);
+    msg_gimv("graph/$filename") if($printer=~/show/);
 
     $tmp="entropy";
     @entropy=($position,$tmp,$upstream,$downstream-1,@entropy);
@@ -400,14 +408,18 @@ sub base_entropy{
   Usage:
     ($position, "IC", $upstream-length, $downstream-length, @entropyvalues) = base_information_content($genome);
 
-  Options:
-     -position   either "start" (around start codon) or "end" (around stop codon)
-                 to create the PWM (default: "start");
-     -PatLen     length of oligomer to count (default: 3)
-     -upstream   length upstream of specified position to create PWM (default: 30)
-     -downstream length downstream of specified position to create PWM (default: 30)
-     -filename   outfile file name (default: consensus.csv)
-     -output     "g" for graph, "f" for file, "show" for display
+ Options:
+    -output     "stdout" for standard output
+                "f" for file output in directory "data",
+                "g" for graph output in directory "graph",
+                "show" for graph output and display (default: "show")
+    -filename   output filename (default: "base_information_content.png" for -output=>"g",
+                                          "base_information_content.csv" for -output=>"f")
+    -position   either "start" (around start codon) or "end" (around stop codon)
+                to create the PWM (default: "start");
+    -PatLen     length of oligomer to count (default: 3)
+    -upstream   length upstream of specified position to create PWM (default: 30)
+    -downstream length downstream of specified position to create PWM (default: 30)
 
   Author: 
     Koya Mori(mory@g-language.org)
@@ -418,7 +430,7 @@ sub base_entropy{
 =cut
 
 sub base_information_content{
-    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
+    &opt_default(output=>"show",filename=>"base_information_content.png",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -438,11 +450,13 @@ sub base_information_content{
     my $tmp;
     my @y;
     
-    $hash=base_counter($gb,-output=>"n",-filename=>$filename,-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
+    $hash=base_counter($gb,-output=>"n",-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
     $downstream++;
 
     if($printer=~/f/){
-	open(FILE,">>$filename");
+        $filename =~ s/\.png$/\.csv/;
+        mkdir ("data", 0777);
+        open(FILE,">>data/$filename");
     }
 
     for(my $i=$upstream;$i>-$downstream;$i--){
@@ -473,7 +487,7 @@ sub base_information_content{
 	if($printer=~/f/){
 	    printf FILE "%d,%.5f\n",-$i ,$IC[$upstream-$i];
 	}
-	if($printer!~/[fn]/){
+	if($printer eq 'stdout'){
 	    &msg_send(sprintf("%d\t%.5f\n",-$i ,$IC[$upstream-$i]));
 	}
 	push(@y,-$i);
@@ -482,10 +496,10 @@ sub base_information_content{
     close(FILE);
 
     if($printer=~/g/ || $printer=~/show/){
-	&G::Tools::Graph::_UniMultiGrapher(\@y,\@IC,-filename=>"base_information_content.png",-x=>"position",-y=>"information_content",-title=>"information_content");
+	&G::Tools::Graph::_UniMultiGrapher(\@y,\@IC,-filename=>$filename,-x=>"position",-y=>"information_content",-title=>"information_content");
     }
 
-    msg_gimv('graph/base_information_content.png') if($printer=~/show/);
+    msg_gimv("graph/$filename") if($printer=~/show/);
 
     $tmp="IC";
     @IC=($position,$tmp,$upstream,$downstream-1,@IC);
@@ -505,13 +519,17 @@ sub base_information_content{
     ($position, "RE", $upstream-length, $downstream-length, @entropyvalues) = base_relative_entropy($genome);
 
   Options:
-     -position   either "start" (around start codon) or "end" (around stop codon)
-                 to create the PWM (default: "start");
-     -PatLen     length of oligomer to count (default: 3)
-     -upstream   length upstream of specified position to create PWM (default: 30)
-     -downstream length downstream of specified position to create PWM (default: 30)
-     -filename   outfile file name (default: consensus.csv)
-     -output     "g" for graph, "f" for file, "show" for display
+    -output     "stdout" for standard output
+                "f" for file output in directory "data",
+                "g" for graph output in directory "graph",
+                "show" for graph output and display (default: "show")
+    -filename   output filename (default: "base_relative_entropy.png" for -output=>"g",
+                                          "base_relative_entropy.csv" for -output=>"f")
+    -position   either "start" (around start codon) or "end" (around stop codon)
+                to create the PWM (default: "start");
+    -PatLen     length of oligomer to count (default: 3)
+    -upstream   length upstream of specified position to create PWM (default: 30)
+    -downstream length downstream of specified position to create PWM (default: 30)
 
   Author: 
     Koya Mori(mory@g-language.org)
@@ -522,7 +540,7 @@ sub base_information_content{
 =cut
 
 sub base_relative_entropy{
-    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
+    &opt_default(output=>"show",filename=>"base_relative_entropy.png",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -541,11 +559,13 @@ sub base_relative_entropy{
     my $tmp;
     my @y;
     
-    $hash=base_counter($gb,-output=>"n",-filename=>$filename,-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
+    $hash=base_counter($gb,-output=>"n",-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
     $downstream++;
 
     if($printer=~/f/){
-	open(FILE,">>$filename");
+        $filename =~ s/\.png$/\.csv/;
+        mkdir ("data", 0777);
+        open(FILE,">>data/$filename");
     }
 
     for(my $i=$upstream;$i>-$downstream;$i--){
@@ -567,7 +587,7 @@ sub base_relative_entropy{
 	if($printer=~/f/){
 	    printf FILE "%d,%.5f\n",-$i ,$RE[$upstream-$i];
 	}
-	if($printer!~/[fn]/){
+	if($printer eq 'stdout'){
 	    &msg_send(sprintf("%d\t%.5f\n",-$i ,$RE[$upstream-$i]));
 	}
 	push(@y,-$i);
@@ -575,10 +595,10 @@ sub base_relative_entropy{
     close(FILE);
 
      if($printer=~/g/ || $printer=~/show/){
-	&G::Tools::Graph::_UniMultiGrapher(\@y,\@RE,-filename=>"base_relative_entropy.png",-x=>"position",-y=>"relative_entropy",-title=>"relative_entropy");
+	&G::Tools::Graph::_UniMultiGrapher(\@y,\@RE,-filename=>$filename,-x=>"position",-y=>"relative_entropy",-title=>"relative_entropy");
     }
 
-    msg_gimv('graph/base_relative_entropy.png') if($printer=~/show/);
+    msg_gimv("graph/$filename") if($printer=~/show/);
    
     $tmp="RE";
     @RE=($position,$tmp,$upstream,$downstream-1,@RE);
@@ -600,14 +620,17 @@ sub base_relative_entropy{
     Here the $structure is a reference to the array of hashes, intended for 
     internal usage.
 
+
   Options:
-     -limit      rank threshold for showing the conserved oligomer (default: 5)
-     -position   either "start" (around start codon) or "end" (around stop codon)
-                 to create the PWM (default: "start");
-     -PatLen     length of oligomer to count (default: 3)
-     -upstream   length upstream of specified position to create PWM (default: 30)
-     -downstream length downstream of specified position to create PWM (default: 30)
-     -output     "stdout" for standard output, "f" for file
+    -output     "stdout" for standard output (default: "stdout")
+                "f" for file output in directory "data"
+    -filename   output filename (default: "base_z_value.csv")
+    -limit      rank threshold for showing the conserved oligomer (default: 5)
+    -position   either "start" (around start codon) or "end" (around stop codon)
+                to create the PWM (default: "start");
+    -PatLen     length of oligomer to count (default: 3)
+    -upstream   length upstream of specified position to create PWM (default: 30)
+    -downstream length downstream of specified position to create PWM (default: 30)
 
   Author: 
     Koya Mori(mory@g-language.org)
@@ -618,7 +641,7 @@ sub base_relative_entropy{
 =cut
 
 sub base_z_value{
-    &opt_default(limit=>5,output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
+    &opt_default(limit=>5,output=>"stdout",filename=>"base_z_value.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -641,11 +664,12 @@ sub base_z_value{
     my @key;
     my $c;
     
-    $hash=base_counter($gb,-output=>"n",-filename=>$filename,-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
-    $downstream++;    
+    $hash=base_counter($gb,-output=>"n",-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
+    $downstream++;
 
-    if($printer eq "f"){
-	open(FILE,">>$filename");
+    if($printer=~/f/){
+        mkdir ("data", 0777);
+        open(FILE,">>data/$filename");
     }
 
     for(my $i=$upstream;$i>-$downstream;$i--){
@@ -744,8 +768,8 @@ sub base_individual_information_matrix{
     my @IIM;
     my %q;
 
-    $hash=base_counter($gb,-output=>"n",-filename=>$filename,-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
-    $downstream++;    
+    $hash=base_counter($gb,-output=>"n",-position=>$position,-PatLength=>$PatLen,-upstream=>$upstream,-downstream=>$downstream);
+    $downstream++;
 
     for(my $i=$upstream;$i>-$downstream;$i--){
 	$hgenome=0;
@@ -862,7 +886,7 @@ sub consensus_z{
     my $high_z = opt_val("high");
     my $low_z = opt_val("low");
     my $filename = opt_val("filename");
-    $filename = 'consensus_z.csv' if (opt_val("output") eq "f");
+    $filename =~ s/\.png$/\.csv/ if (opt_val("output") eq "f");
     my $output = opt_val("output");
     my @inseq = @$ref;
     my ($i,$tmp,$outseq,@seq,@array,%nuc_table, @out, @pos);
