@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GCskew.pm,v 1.3 2002/08/16 15:03:36 gaou Exp $
@@ -50,6 +50,7 @@ our @EXPORT = qw(
 	     genomicskew
 	     geneskew
 	     gcwin
+	     coding_density
 	     leading_strand
 	     query_strand
 	     query_arm
@@ -65,7 +66,7 @@ our @EXPORT = qw(
 );
 
 
-#__DATA__
+__DATA__
 
 #:::::::::::::::::::::::::::::::::
 #       Perldoc
@@ -533,6 +534,9 @@ sub leading_strand {
   References:
     1. Arakawa K, Tomita M (2007) "The GC skew index: a measure of genomic compositional 
        asymmetry and the degree of replicational selection", Evolutionary Bioinformatics, 3:145-154.
+    2. Arakawa K, et al. (2009) "Quantitative analysis of replication-related mutation
+       and selection pressures in bacterial chromosomes and plasmids using generalised GC skew index",
+       BMC Genomics, 10:640.
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
@@ -798,7 +802,7 @@ sub cum_gcskew {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
  History:
-   200110216-01 initial posting
+   20110216-01 initial posting
 
 =cut
 
@@ -1162,6 +1166,122 @@ sub gcwin  {
 }
 
 
+=head2 coding_density
+
+ Name: coding_density   -   calculate and plot the coding density along the given genome
+
+ Description:
+   This program calculates and graphs the coding density along the given genome.
+   To obtain the overall genomic coding density (percentage of nucleotides in ORFs), 
+   run the function in scalar context. 
+
+ Usage: 
+    array  @coding_density = coding_density(G instance); #list of windowed values
+    scalar $coding_density = coding_density(G instance); #percentage in a genome
+
+ Options:
+   -window      window size to observe (default: 10000)
+   -gene        plot gene counts instead of coding density
+   -cumulative  plot cumulative graph
+   -output      f for file output in directory "data", 
+                g for graph output in directory "graph",
+                show for graph output and display (default: "show")
+   -filename    output filename (default: "coding_density.png" for -output=>"g",
+                                          "coding_density.csv" for -output=>"f")
+
+ Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+ History:
+   20120619-01 initial posting
+
+=cut
+
+
+
+
+sub coding_density  {
+    &opt_default(window=>10000, output=>"show", filename=>"coding_density.png", gene=>0, cumulative=>0);
+    my @args = opt_get(@_);
+
+    my $gb         = opt_as_gb(shift @args);
+    my $window     = opt_val("window");
+    my $filename   = opt_val("filename");
+    my $gene       = opt_val("gene");
+    my $cumulative = opt_val("cumulative");
+    $filename      =~ s/\.png$/\.csv/ if (opt_val("output") eq 'f');
+    my $opt        = opt_val("output");
+
+    my (@pos, @num);
+    my $j;
+
+    for my $cds ($gb->cds()){
+	if(length($gb->{$cds}->{join})){
+	    for my $segment (split(/,/, $gb->{$cds}->{join})){
+		$segment =~ s/c//g;
+		my ($start, $end) = split(/\.\./, $segment, 2);
+		substr($gb->{SEQ}, $start - 1, $end - $start + 1) = "P" x ($end - $start + 1);
+	    }
+	}else{
+	    substr($gb->{SEQ}, $gb->{$cds}->{start} - 1, $gb->{$cds}->{end} - $gb->{$cds}->{start} + 1) = 
+		"P" x ($gb->{$cds}->{end} - $gb->{$cds}->{start} + 1);
+	}
+    }
+
+    if($gene){
+	for (my $i = 0; $i + $window <= length($gb->{SEQ}); $i += $window){
+	    push(@pos, $i);
+	    
+	    my $j = 0;
+	    for my $cds ($gb->cds()){
+		$j ++ if ($i < $gb->{$cds}->{start} && $gb->{$cds}->{start} <= $i + $window);
+	    }
+	    push(@num, $j);
+	}
+    }else{
+	for (my $i = 0; $i + $window <= length($gb->{SEQ}); $i += $window){
+	    my $seq = substr($gb->{SEQ}, $i, $window);
+	    my $P = $seq =~ tr/P/P/;
+	    push(@pos, $i);
+	    push(@num, $P/$window);
+	}
+    }
+
+    @num = cumulative(@num, -mean=>1) if($cumulative);
+
+    my $title = "coding density";
+    $title = 'cumulative ' . $title;
+    $title .= ' (gene count)' if ($gene);
+
+    if ($opt eq 'g' || $opt eq 'show'){
+	mkdir ("graph", 0777);
+	grapher(
+		\@pos, \@num,
+		-x=>"bp", -y=>$title, 
+		-filename=>$filename,
+		-title=>$title, -style=>"lines", -type=>"columns",
+		-output=>$opt
+		);
+    }elsif ($opt eq 'f'){
+	my $j = 0;
+	mkdir ("data", 0777);
+	open(OUT, ">data/" . $filename);
+	print OUT "location,$title\n";
+	for ($j = 0; $j <= $#pos; $j++){
+	    print OUT $pos[$j], ",", $num[$j], "\n";
+	}
+	close(OUT);
+    }
+    
+    if(wantarray()){
+	return @num;
+    }else{
+	my $n = $gb->{SEQ} =~ tr/P/P/;
+	return ($n / length($gb->{SEQ}));
+    }
+}
+
+
 
 =head2 query_strand
 
