@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Util.pm,v 1.1.1.1 2002/04/02 20:25:43 gaou Exp $
@@ -38,6 +38,7 @@ use G::Seq::GCskew;
 use G::DB::SDB;
 
 use GD;
+#use GD::SVG;
 use SVG;
 
 use strict;
@@ -156,6 +157,7 @@ my $pi = 4 * atan2(1,1);
     NULL = circular_map($gb);
 
  Options:
+    -filename       output filename (default:'circular_map.svg')  
     -gmap           1 to create Google Map view, 0 to only generate PNG image
                     (default:1)
     -optionalText   optional text to describe the genome.
@@ -164,6 +166,7 @@ my $pi = 4 * atan2(1,1);
     Nobuhiro Kido (t07505nk@sfc.keio.ac.jp)
 
   History:
+   20120824-01 added -filename option
    20100318-01 major update including support for linear chromosomes,
                change of gene color by strand in circular genomes,
                addition of accession number and optional text,
@@ -175,10 +178,11 @@ my $pi = 4 * atan2(1,1);
 
 
 sub circular_map{
-    opt_default("gmap"=>1);
-    my @args = opt_get(@_);
-    my $gb   = opt_as_gb(shift @args);
-    my $gmap = opt_val("gmap");
+    opt_default("gmap"=>1, "filename"=>"circular_map.svg", "output"=>"g");
+    my @args     = opt_get(@_);
+    my $gb       = opt_as_gb(shift @args);
+    my $filename = opt_val("filename");
+    my $gmap     = opt_val("gmap");
     my $optionalText = opt_val("optionalText");
     mkdir( "data",  0777 );
     mkdir( "graph", 0777 );
@@ -747,12 +751,12 @@ sub circular_map{
     )->cdata("GC-skew");
 
     close PWRITER;
-    open( WRITER, ">./graph/" . $wfile . ".svg" );
+    open( WRITER, ">./graph/" . $filename );
     print WRITER $svg->xmlify;
     close WRITER;
 
     if($gmap){
-	generateGMap("graph/$wfile.svg");
+	generateGMap("graph/$filename");
     }
 }
 
@@ -1616,6 +1620,7 @@ sub genome_map2 {
  Options:
     -width          image width (default: 8192)
     -height         image height (default: 8192)
+    -filename       output filename (default: 'genome_map3.png')
     -output         "g" for graph, "show" for display (default: "show") 
     -datafilename   coordinate file name (default: "[accessionNumber].coord")
     -gmap           1 to create Google Map view, 0 to only generate PNG image
@@ -1625,6 +1630,7 @@ sub genome_map2 {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+   20120824-01 added -filename option
    20100319-01 added support for exons
    20071008-01 added -gmap option
    20070628-01 added -datafilename option
@@ -1633,24 +1639,24 @@ sub genome_map2 {
 =cut
 
 sub genome_map3 {
-    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>'', gmap=>1);
-    my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
-    my $acnum = $gb->{LOCUS}->{id};
+    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>'', gmap=>1, filename=>"genome_map3.png");
+    my @args         = opt_get(@_);
+    my $gb           = opt_as_gb(shift @args);
+    my $acnum        = $gb->{LOCUS}->{id};
     my $datafilename = opt_val("datafilename") || $gb->id() . '.coord';
-    my $output = opt_val("output");
-    my $filename;
-    my $topmargin = 50;
-    my $sidemargin = 150;
-    my $hblock = 100;
-    my $vblock = 10;
-    my $width = opt_val("width");
-    my $height = opt_val("height");
-    my $gmap = opt_val("gmap");
+    my $output       = opt_val("output");
+    my $filename     = opt_val("filename");
+    my $topmargin    = 50;
+    my $sidemargin   = 150;
+    my $hblock       = 100;
+    my $vblock       = 10;
+    my $width        = opt_val("width");
+    my $height       = opt_val("height");
+    my $gmap         = opt_val("gmap");
 
     my $numHBlock = int(($width - $sidemargin - 50) / $hblock);
     my $numVBlock = int(($height - $topmargin - 50) / ($vblock * 5));
-    my $window = int(length($gb->{SEQ}) / ($numHBlock * $numVBlock * $hblock)) + 1;
+    my $window    = int(length($gb->{SEQ}) / ($numHBlock * $numVBlock * $hblock)) + 1;
 
     msg_error("Image size: $width x $height pixels.\nUsing Window size = $window bp to fit the whole genome...\n\n");
     msg_error("Map generation will take a few minutes. Please make yourself comfortable until the image is generated ;-\)\n\n") if ($gmap);
@@ -1929,12 +1935,12 @@ sub genome_map3 {
     }
     close(OUT);
 
-    open(IMAGE, '>graph/' . $acnum . '.png');
+    open(IMAGE, '>graph/' . $filename);
     binmode IMAGE;
     print IMAGE $im->png;
     close(IMAGE);
 
-    generateGMap("graph/$acnum.png") if($gmap);
+    generateGMap("graph/$filename") if($gmap);
 }
 
 
@@ -2251,6 +2257,7 @@ sub plasmid_map{
     NULL = dnawalk($gb);
 
  Options:
+    -filename       output filename (default:'dnawalk.png')  
     -gmap           1 to create Google Map view, 0 to only generate PNG image
                     (default:1)
 
@@ -2258,6 +2265,7 @@ sub plasmid_map{
     Keita Ikegami (t06056ki@sfc.keio.ac.jp)
 
   History:
+   20120824-01 added -filename option
    20080414-01 added gmap option
    20071023-01 initial posting
 
@@ -2265,13 +2273,14 @@ sub plasmid_map{
 
 
 sub dnawalk{
-    opt_default("gmap"=>1);
+    opt_default("gmap"=>1, "filename"=>'dnawalk.png', "output"=>"g");
     my @args = opt_get(@_);
 
     my $gb = opt_as_gb(shift @args);
     my $name = $gb->{LOCUS}->{id};
     my $segments = 10;
     my $gmap = opt_val("gmap");
+    my $filename = opt_val("filename");
 
     mkdir("data", 0777);
     mkdir("graph", 0777);
@@ -2390,12 +2399,12 @@ sub dnawalk{
     }
     close DATA;   
     
-    open NEW, ">./graph/" . $gb->{LOCUS}->{id} . ".png"; #automatically name the file
+    open NEW, ">./graph/" . $filename;
     binmode NEW;
     print NEW $im->png;
     close NEW;
 
-    generateGMap("graph/". $gb->{LOCUS}->{id} . ".png") if($gmap);
+    generateGMap("graph/$filename") if($gmap);
 }
 
 1;
