@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: PatSearch.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -27,11 +27,13 @@
 package G::Seq::PatSearch;
 
 use SubOpt;
+use Rcmd;
 use G::Messenger;
 use G::Tools::Graph;
 use G::Tools::GMap;
 use G::Tools::Statistics;
 use G::Seq::Primitive;
+use File::Temp;
 
 use strict;
 use base qw(Exporter);
@@ -269,16 +271,15 @@ sub markov {
    -seq        sequence (default: '')
                '' when using whole genome sequence; i.e. -seq=>$gb->{SEQ}
    -memo       memo (default: '')
+   -header     include (1) or exclude (0) variable names (default: 1)
 
   Author: 
    Haruo Suzuki (haruo@g-language.org)
 
   History:
-   20101010 initial posting
+   20120617 added -header option
    20110222 added -bothstrand, -oe, -seq, and -memo options
-
-  Informations stored in the G instance:
-    $gb->{"header_signature"}
+   20101010 initial posting
 
   References:
    Campbell A et al. (1999) Proc Natl Acad Sci U S A. 96(16):9184-9.
@@ -289,7 +290,7 @@ sub markov {
 =cut
 
 sub signature {
-    &opt_default(output=>'stdout', filename=>'signature.csv', wordlength=>2, bothstrand=>1, oe=>1, seq=>'', memo=>'');
+    &opt_default(output=>'stdout', filename=>'signature.csv', wordlength=>2, bothstrand=>1, oe=>1, seq=>'', memo=>'', header=>1);
     my @args       = opt_get(@_);
     my $gb         = opt_as_gb(shift @args);
     my $output     = opt_val("output");
@@ -299,6 +300,7 @@ sub signature {
     my $oe         = opt_val("oe");
     my $seq        = opt_val("seq");
     my $memo       = opt_val("memo");
+    my $header     = opt_val("header");
 
     my ($total, %mono, %obs, %exp, %val);
 
@@ -371,17 +373,13 @@ sub signature {
     }
 
     if($output eq 'f'){
-	#mkdir ("data", 0777);
-	#open(FILE,">>data/$filename");
-	open(my $fh, ">>$filename");
-	unless($gb->{"header_signature"}){
-	    $gb->{"header_signature"} = 1;
-	    foreach(@allkey){ print $fh "$_,"; }
-	    print $fh "memo\n";
-	}
-	foreach(@allkey){ print $fh (exists $val{$_}) ? "$val{$_}," : "0,"; }
-	print $fh "$memo\n";
-	close($fh);
+	mkdir ("data", 0777);
+	open(OUT, ">>data/$filename");
+	if($header){ print OUT join(',', @allkey), ",memo\n"; }
+	my @tmp;
+	foreach (@allkey){ push(@tmp, $val{$_} || 0); }
+	print OUT join(',', @tmp), ",$memo\n";
+	close(OUT);
     }
 
     $gb->{signature} = \%val; # input in the G instance
@@ -418,8 +416,8 @@ sub signature {
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20120617 use File::Temp;
    20101010 initial posting
-   20110222 minor change
 
   References:
    Campbell A et al. (1999) Proc Natl Acad Sci U S A. 96(16):9184-9.
@@ -433,33 +431,44 @@ sub signature {
 sub signature_dist {
     &opt_default(output=>'stdout', wordlength=>2, window=>5000, step=>5000, method=>'mahalanobis');
     my @args       = opt_get(@_);
-    my $query      = opt_as_gb(shift @args);
-    my $sbjct      = opt_as_gb(shift @args);
+    my $gb1 = shift @args;
+    my $gb2 = shift @args;
+    unless(defined($gb1) && defined($gb2)){ msg_error("At least two different genomes should be provided as follows. &signature_dist(\$gb_plasmid, \$gb_chromosome);\n"); return; }
+    my $query      = opt_as_gb($gb1);
+    my $sbjct      = opt_as_gb($gb2);
     my $output     = opt_val("output");
     my $wordlength = opt_val("wordlength");
     my $window     = opt_val("window");
     my $step       = opt_val("step");
     my $method     = opt_val("method");
-    my $rand       = time() . rand(1000);
- 
-#    system('echo -n > query.csv > sbjct.csv');
 
-    &signature($query, -output=>"f", -filename=>"/tmp/query$rand.csv", -wordlength=>$wordlength);
+    my $fh_query = File::Temp->new; my $fname_query = $fh_query->filename;
+    my $fh_sbjct = File::Temp->new; my $fname_sbjct = $fh_sbjct->filename;
+
+    my $val = &signature($query, -output=>"n", -filename=>$fname_query, -wordlength=>$wordlength);
+    my @allkey = sort keys %$val;
+    print $fh_query join(',',@allkey),"\n";
+    my @tmp;
+    foreach (@allkey){ push(@tmp, $$val{$_} || 0); }
+    print $fh_query join(',', @tmp), "\n";
+    
+    print $fh_sbjct join(',',@allkey),"\n";
     for(my $i = 1; $i <= length($sbjct->{SEQ}) - $window; $i += $step){
 	my $start = $i;
 	my $end = $i + $window - 1;
 	my $seq = $sbjct->get_gbkseq($start, $end);
-	&signature($sbjct, -output=>"f", -filename=>"/tmp/sbjct$rand.csv", -wordlength=>$wordlength, -seq=>$seq);
+	my $val = &signature($sbjct, -output=>"n", -filename=>$fname_sbjct, -wordlength=>$wordlength, -seq=>$seq);
+	my @tmp;
+	foreach (@allkey){ push(@tmp, $$val{$_} || 0); }
+	print $fh_sbjct join(',', @tmp), "\n";
     }
-
+    
     my $rcmd = new Rcmd;
     my @result = $rcmd->exec(
 			     qq|
 			     method = "$method"
-			     X = read.csv("/tmp/query$rand.csv")
-			     X = X[,-ncol(X)]
-			     Y = read.csv("/tmp/sbjct$rand.csv")
-			     Y = Y[,-ncol(Y)]
+			     X = read.csv("$fname_query")
+			     Y = read.csv("$fname_sbjct")
 			     if(method == "mahalanobis"){ 
                                  library(MASS);
                                  dX = try(mahalanobis(X, apply(Y,2,mean), ginv(var(Y)), inverted=TRUE), silent=TRUE);
@@ -477,13 +486,9 @@ sub signature_dist {
 			     as.vector(c(D, P))
 			     |
 			     );
-
-    unlink("/tmp/query$rand.csv");
-    unlink("/tmp/sbjct$rand.csv");
-#    print "@result \n";
+    #print "@result \n";
     my $distance = sprintf "%.4f", shift @result;
     my $p_value = sprintf "%.4f", shift @result;
-#    system('rm query.csv sbjct.csv');
     &msg_send("\n$wordlength-mer genomic signature difference:\n $method distance = $distance\n p-value = $p_value\n") if ($output eq 'stdout');
     return ($distance, $p_value);
 }
@@ -1066,7 +1071,7 @@ sub baseParingTest {
   Options:
     -nucleotide    nucleotide to search (default:aa)
     -window        window size to seek periodicity (default:50)
-    -filename      output filename (default:aa_frequency.png)
+    -filename      output filename (default:nucleotide_periodicity.png)
     -output        "g" for graph file output only,
                    "show" for graph file output and display.
                    (default: show)
@@ -1086,7 +1091,7 @@ sub baseParingTest {
 
 
 sub nucleotide_periodicity {
-    opt_default("nucleotide"=>"aa", "window"=>50, "filename"=>"aa_frequency.png", "output"=>"show");
+    opt_default("nucleotide"=>"aa", "window"=>50, "filename"=>"nucleotide_periodicity.png", "output"=>"show");
     my @argv = opt_get(@_);
     my $gb = opt_as_gb(shift @argv);
     my $nuc = opt_val("nucleotide");
@@ -1105,9 +1110,19 @@ sub nucleotide_periodicity {
 	}
     }
 
-    _UniMultiGrapher([0..($window - 1)], \@data, -filename=>$filename);
-    msg_gimv("graph/$filename") if ($output eq 'show');
-
+    if($output eq 'f'){
+	$filename = 'nucleotide_periodicity.csv' if ($filename eq 'nucleotide_periodicity.png');
+	mkdir ("data", 0777);
+	open(OUT, ">>data/$filename");
+	print OUT "window,value\n";
+	for my $win (0..($window-1)){
+	    print OUT $win, ',', $data[$win], "\n";
+	}
+	close(OUT);
+    }else{
+	_UniMultiGrapher([0..($window - 1)], \@data, -filename=>$filename, -title=>"Periodicity of $nuc", -x=>"position", -y=>"count");
+	msg_gimv("graph/$filename") if ($output eq 'show');
+    }
     return @data;
 }
 
