@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Primitive.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -84,6 +84,110 @@ my %CodonTable = (
 		  );
 
 
+# http://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi
+# 2. The Vertebrate Mitochondrial Code
+my %CodonTable2 = %CodonTable;
+$CodonTable2{'aga'} = '/';
+$CodonTable2{'agg'} = '/';
+$CodonTable2{'ata'} = 'M';
+$CodonTable2{'tga'} = 'W';
+
+# 3. The Yeast Mitochondrial Code
+my %CodonTable3 = %CodonTable;
+$CodonTable3{'ata'} = 'M';
+$CodonTable3{'ctt'} = 'T';
+$CodonTable3{'ctc'} = 'T';
+$CodonTable3{'cta'} = 'T';
+$CodonTable3{'ctg'} = 'T';
+$CodonTable3{'tga'} = 'W';
+$CodonTable3{'cga'} = '/'; # absent
+$CodonTable3{'cgc'} = '/'; # absent
+
+# 4. The Mold, Protozoan, and Coelenterate Mitochondrial Code and the Mycoplasma/Spiroplasma Code
+my %CodonTable4 = %CodonTable;
+$CodonTable4{'tga'} = 'W';
+
+# 5. The Invertebrate Mitochondrial Code
+my %CodonTable5 = %CodonTable;
+$CodonTable5{'aga'} = 'S';
+$CodonTable5{'agg'} = 'S';
+$CodonTable5{'ata'} = 'M';
+$CodonTable5{'tga'} = 'W';
+
+# 6. The Ciliate, Dasycladacean and Hexamita Nuclear Code
+my %CodonTable6 = %CodonTable;
+$CodonTable6{'taa'} = 'Q';
+$CodonTable6{'tag'} = 'Q';
+
+# 9. The Echinoderm and Flatworm Mitochondrial Code#
+my %CodonTable9 = %CodonTable;
+$CodonTable9{'aaa'} = 'N';
+$CodonTable9{'aga'} = 'S';
+$CodonTable9{'agg'} = 'S';
+$CodonTable9{'tga'} = 'W';
+
+# 10. The Euplotid Nuclear Code
+my %CodonTable10 = %CodonTable;
+$CodonTable10{'tga'} = 'C';
+
+# 11. The Bacterial, Archaeal and Plant Plastid Code
+my %CodonTable11 = %CodonTable;
+
+# 12. The Alternative Yeast Nuclear Code
+my %CodonTable12 = %CodonTable;
+$CodonTable12{'ctg'} = 'S';
+
+# 13. The Ascidian Mitochondrial Code
+my %CodonTable13 = %CodonTable;
+$CodonTable13{'aga'} = 'G';
+$CodonTable13{'agg'} = 'G';
+$CodonTable13{'ata'} = 'M';
+$CodonTable13{'tga'} = 'W';
+
+# 14. The Alternative Flatworm Mitochondrial Code
+my %CodonTable14 = %CodonTable;
+$CodonTable14{'aaa'} = 'N';
+$CodonTable14{'aga'} = 'S';
+$CodonTable14{'agg'} = 'S';
+$CodonTable14{'taa'} = 'Y';
+$CodonTable14{'tga'} = 'W';
+
+# 16. Chlorophycean Mitochondrial Code
+my %CodonTable16 = %CodonTable;
+$CodonTable16{'tag'} = 'L';
+
+# 21. Trematode Mitochondrial Code
+my %CodonTable21 = %CodonTable;
+$CodonTable21{'tga'} = 'W';
+$CodonTable21{'ata'} = 'M';
+$CodonTable21{'aga'} = 'S';
+$CodonTable21{'agg'} = 'S';
+$CodonTable21{'aaa'} = 'N';
+
+# 22. Scenedesmus obliquus Mitochondrial Code
+my %CodonTable22 = %CodonTable;
+$CodonTable22{'tca'} = '/';
+$CodonTable22{'tag'} = 'L';
+
+# 23. Thraustochytrium Mitochondrial Code
+ # it contains an additional stop codon (TTA) and also has a different set of start codons.
+my %CodonTable23 = %CodonTable;
+$CodonTable23{'tta'} = '/';
+
+# 24. Pterobranchia Mitochondrial Code
+my %CodonTable24 = %CodonTable;
+$CodonTable24{'aga'} = 'S';
+$CodonTable24{'agg'} = 'K';
+$CodonTable24{'tga'} = 'W';
+
+# 25. Candidate Division SR1 and Gracilibacteria Code
+my %CodonTable25 = %CodonTable;
+$CodonTable25{'tga'} = 'G';
+
+my @CodonTables = ('', \%CodonTable2, \%CodonTable3, \%CodonTable4, \%CodonTable5, \%CodonTable6, '', '', \%CodonTable9, \%CodonTable10, \%CodonTable11, \%CodonTable12,
+		   \%CodonTable13, \%CodonTable14, '', \%CodonTable16, '', '', '', '', \%CodonTable21, \%CodonTable22, \%CodonTable23, \%CodonTable24, \%CodonTable25);
+
+
 #::::::::::::::::::::::::::::::
 #    Let the code begin...
 #::::::::::::::::::::::::::::::
@@ -109,9 +213,37 @@ my %CodonTable = (
     # to use custom codon table
     $amino = translate($nucleotide, %customCodonTable);
 
+    # to use alternative codon table (2, Vertebrate Mitochondrial Code)
+    $amino = translate($nucleotide, -table=>2);
+
   Options:
     Custom codon table can be optionally used (see above).
 
+    Alternative Codon Tables can be used with:
+    -table   number of alternative codon table (default: 1)
+             Supported tables are listed below:
+
+      1. The Standard Code
+      2. The Vertebrate Mitochondrial Code
+      3. The Yeast Mitochondrial Code
+      4. The Mold, Protozoan, and Coelenterate Mitochondrial Code and the Mycoplasma/Spiroplasma Code
+      5. The Invertebrate Mitochondrial Code
+      6. The Ciliate, Dasycladacean and Hexamita Nuclear Code
+      9. The Echinoderm and Flatworm Mitochondrial Code
+      10. The Euplotid Nuclear Code
+      11. The Bacterial, Archaeal and Plant Plastid Code
+      12. The Alternative Yeast Nuclear Code
+      13. The Ascidian Mitochondrial Code
+      14. The Alternative Flatworm Mitochondrial Code
+      16. Chlorophycean Mitochondrial Code
+      21. Trematode Mitochondrial Code
+      22. Scenedesmus obliquus Mitochondrial Code
+      23. Thraustochytrium Mitochondrial Code
+      24. Pterobranchia Mitochondrial Code
+      25. Candidate Division SR1 and Gracilibacteria Code
+  
+      see http://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi for details of the tables.
+
   REST: 
       http://rest.g-language.org/translate/ctggtg
 
@@ -119,6 +251,7 @@ my %CodonTable = (
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20141114-01 added alternative codon tables
     20091203-01 added cutom codon table option
     20010727-01 initial posting
 
@@ -126,10 +259,14 @@ my %CodonTable = (
 
 
 sub translate {
-    my $seq = lc(shift);
+    my @args  = opt_get(@_);
+    my $seq   = lc(shift @args);
+    my $table = opt_val("table"); 
     my %option = @_;
 
     my %codons = %CodonTable;
+    %codons = %{$CodonTables[$table - 1]} if(length($CodonTables[$table - 1]));
+
     foreach my $key (%option){
 	$codons{$key} = $option{$key};
     }
@@ -248,9 +385,14 @@ sub splitprintseq {
                  if the first argument is an instance of new G(), 
                  $gb->{LOCUS}->{id} is used as default
     -length      number of characters per line (default: 60)
+    -amino       when the given sequence is a G-language object, outputs the "translation" 
+                 feature of all CDS as multi-FASTA format.
     -filter      reference to a list of FASTA IDs to output, when a hash of
                  multiple sequences are given. Only the sequences with IDs
                  supplied by this option is returned.
+    -rfilter     reference to a list of FASTA IDs to NOT output, when a hash of
+                 multiple sequences are given. Only the sequences not matching to
+                 the supplied list  is returned.
     -filename    output filename (default: "out.fasta")
     -output      "f" to output to file, and return the fasta as a string.
                  "return" to just return the fasta as a string (default: "return")
@@ -259,6 +401,8 @@ sub splitprintseq {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20120306-01 added -amino option
+    20110504-01 added -rfilter option
     20110428-01 added support for multi-fasta given as a hash, and -filter option
     20090312-01 changed the deafult behavior to return fasta as a string as well as outputting to a file
     20070612-01 added -output option
@@ -275,16 +419,32 @@ sub to_fasta{
     my $length   = opt_val("length");
     my $output   = opt_val("output");
     my $filter   = opt_val("filter");
+    my $rfilter  = opt_val("rfilter");
+    my $amino    = opt_val("amino");
     my $sequence = $gb;
 
-    my %flt;
+    if($filename ne 'out.fasta'){
+	$output = 'f';
+    }
+
+    my (%flt, %rflt);
     if(length($filter)){
 	map{$flt{$_}++} @$filter;
+    }elsif(length($rfilter)){
+	map{$rflt{$_}++} @$rfilter;
     }
 	
     if(length $gb->{LOCUS}->{id}){
-	$name = $gb->{LOCUS}->{id} if ($name eq 'sequence');
-	$sequence  = $gb->{SEQ};
+	if($amino){
+	    my %tmp;
+	    for my $cds ($gb->cds()){
+		$tmp{$gb->{$cds}->{gene}} = $gb->{$cds}->{translation};
+	    }
+	    @args = %tmp;
+	}else{
+	    $name = $gb->{LOCUS}->{id} if ($name eq 'sequence');
+	    $sequence  = $gb->{SEQ};
+	}
     }
     die("Error: No sequence passed to to_fasta") unless(length($sequence));
     
@@ -295,6 +455,8 @@ sub to_fasta{
 	for my $id (sort keys %seq){
 	    if(length($filter)){
 		next unless($flt{$id});
+	    }elsif(length($rfilter)){
+		next if($rflt{$id});
 	    }
 	    $result .= sprintf ">%s\n%s", $id, splitprintseq($seq{$id}, $length);
 	}
@@ -345,26 +507,36 @@ sub to_fasta{
 
 sub to_fastq{
     SubOpt::opt_default(length=>75, filename=>"out.fastq", name=>"sequence", output=>"return");
-    my @args = SubOpt::opt_get(@_);
-    my $first = shift @args;
+    my @args  = SubOpt::opt_get(@_);
+    my $first = @args[0];
     die("Error: No sequence passed to to_fastq") unless(length($first));
-    my $gb = SubOpt::opt_as_gb($first);
     my $filename = SubOpt::opt_val("filename");
-    my $name = SubOpt::opt_val("name");
-    my $length = SubOpt::opt_val("length");
-    my $output = SubOpt::opt_val("output");
+    my $name     = SubOpt::opt_val("name");
+    my $length   = SubOpt::opt_val("length");
+    my $output   = SubOpt::opt_val("output");
 
-    $name = $gb->{LOCUS}->{id} if($name eq 'sequence' && length $gb->{LOCUS}->{id});
-    $gb->{QUAL} = 'X' x length($gb->{SEQ}) unless(length($gb->{QUAL}));
+    $name = $first->{LOCUS}->{id} if($name eq 'sequence' && length $first->{LOCUS}->{id});
+    $first->{QUAL} = 'X' x length($first->{SEQ}) unless(length($first->{QUAL}));
 
+    my $result;
+    if(scalar(@args) > 1){
+	my %seq = (@args);
+	
+	for my $id (sort keys %seq){
+	    $result .= 	sprintf("\@%s\n%s\+%s\n%s", $id, splitprintseq(uc($seq{$id}), $length), $name, splitprintseq('i' x length($seq{$id}), $length));
+	}
+    }else{
+	$result = sprintf("\@%s\n%s\+%s\n%s", $name, splitprintseq(uc($first->{SEQ}), $length), $name, splitprintseq($first->{QUAL}, $length));
+    }
+    
     if($output eq 'f'){
 	open(OUT, ">$filename") || die($!);
-	printf OUT "\@%s\n%s\+%s\n%s", $name, splitprintseq(uc($gb->{SEQ}), $length), $name, splitprintseq($gb->{QUAL}, $length);
+	printf OUT $result;
 	close(OUT);
-	return sprintf "\@%s\n%s\+%s\n%s", $name, splitprintseq(uc($gb->{SEQ}), $length), $name, splitprintseq($gb->{QUAL}, $length);
-    }else{
-	return sprintf "\@%s\n%s\+%s\n%s", $name, splitprintseq(uc($gb->{SEQ}), $length), $name, splitprintseq($gb->{QUAL}, $length);
     }
+    
+    return $result;
+
 }
 
 
