@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Util.pm,v 1.1.1.1 2002/04/02 20:25:43 gaou Exp $
@@ -33,6 +33,7 @@ use SubOpt;
 use G::Messenger;
 use G::Seq::Primitive;
 use G::Tools::Graph;
+use G::DB::SDB;
 use G::IO;
 use G::IO::Annotation;
 
@@ -40,6 +41,7 @@ use strict;
 use base qw(Exporter);
 use autouse 'Cwd'=>qw(getcwd);
 use SelfLoader;
+use File::ShareDir ':ALL';
 
 our @EXPORT = qw(
 		 seqinfo
@@ -50,6 +52,7 @@ our @EXPORT = qw(
 		 oligomer_translation
 		 filter_cds_by_atg
 		 find_king_of_gene
+		 refseq2gbk
 		 );
 
 
@@ -188,60 +191,96 @@ sub longest_ORF{
 
 =head2 molecular_weight
 
-  Name: molecular_weight   -   calculates the molecular weight of given nucleotide sequence
+  Name: molecular_weight   -   calculates the molecular weight of given sequence or molecule
 
   Description:
-    This method calculates the molecular weight of the given
-    nucleotide sequence, taking into account the hydrogen bonds 
-    between molecules.. Molecular weight used in this method
-    is as follows:
-       A: 313.15
-       T: 304.19
-       G: 329.19
-       C: 289.13
-       N: 308.915
+    This method calculates the molecular weight of the given sequence, either ssDNA, dsDNA, RNA, Peptide,
+    or that of molecular compounds from chemical formula.
+
+    For RNA, 
+      M.W. = (An x 329.2) + (Un x 306.2) + (Cn x 305.2) + (Gn x 345.2) + 159
+    For ssDNA,
+      M.W. = (An x 313.2) + (Tn x 304.2) + (Cn x 289.2) + (Gn x 329.2) + 79.0
+    For dsDNA,
+      M.W. of dsDNA = (nucleotides x 607.4) + 157.9
+
+    For peptides and chemical formula, mass is calculated by the cumulative addition of each 
+    amino acid or atoms. For example, 
+
+      molecular_weight("C6H12O6") #glucose
+
+    returns 180.0633881166
 
  Usage: 
-    -strand      "single" or "double" strand DNA molecule (default:single)
+    $mass = molecular_weight($sequence);
 
  Options:
-   none
+    -type        automatically guessed, but can be optionally specified
+                 to be "ssDNA" (or just "DNA"), "dsDNA", "RNA", "peptide", or "molecule"
 
   Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20130130-01 major update to include DNA, RNA, peptide, and molecules in general
     20011029-01 initial posting
 
 =cut
 
 sub molecular_weight {
-    opt_default(strand=>"single");
-    my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
+    my @args   = opt_get(@_);
+    my $seq    = shift @args;
     my $strand = opt_val("strand");
+    my $type   = lc(opt_val("type"));
 
-    my %mw = ("a", 313.15, "t", 304.19, "g", 329.19, "c", "289.13", "n", 308.915);
-    my $i = 0;
-    my $weight = 0;
-
-    while(substr($gb->{SEQ}, $i, 1) ne ''){
-	if (substr($gb->{SEQ}, $i, 1) =~ /[atgc]/){
-	    $weight += $mw{substr($gb->{SEQ}, $i, 1)};
+    unless(length($type)){
+	if($seq =~ /[0-9]/){
+	    $type = "molecule";
+	}elsif(lc($seq) =~ /^[atgcn]+$/){
+	    $type = "ssdna";
+	}elsif(lc($seq) =~ /^[augcn]+$/){
+	    $type = "rna";
 	}else{
-	    $weight += $mw{"n"};
+	    $type = "peptide";
 	}
-	$i ++;
     }
-    my $double = $weight * 2;
+    $type = "ssdna" if($type eq 'dna');
 
-    msg_send(sprintf "  Molecular Weight of Nucleotides:\n");
-    msg_send(sprintf "    single strand:  %12d\n",$weight); 
-    msg_send(sprintf "    double strand:  %12d\n\n\n",$double); 
+    if($type eq "molecule"){
+	my %atom;
+	for my $line (readFile(dist_file('g-language', 'data/atom.csv')), 1){
+	    my ($atomm, $mass) = split(/,/, $line,2);
+	    $atom{$atomm} = $mass;
+	}
 
-    $weight *= 2 if ($strand eq "double");
+	my $res;
+	while($seq =~ /([A-Z][a-z]*)(\d*)/g){
+	    my $prod = $2 || 1;
+	    $res += $atom{$1} * $prod;
+	}
+	return $res;
+    }else{
+	my (%count, $res);
+	$count{$_}++ for (split(//, lc($seq)));
 
+	if($type eq "peptide"){
+	    require G::Seq::AminoAcid;
+	    my %mass = G::Seq::AminoAcid::mass();
 
-    return $weight;
+	    for my $key (keys %count){
+		$res += $mass{uc($key)} * $count{$key};
+	    }
+
+	    return $res;
+	}elsif($type eq "ssdna"){
+	    return $count{'a'} * 313.2 + $count{'u'} * 304.2 + $count{'t'} * 304.2 + $count{'c'}*289.2 + $count{'g'}*329.2 + $count{'n'} * 303.7 + 79;
+	}elsif($type eq "rna"){
+	    return $count{'a'} * 329.2 + $count{'u'} * 306.2 + $count{'t'} * 306.2 + $count{'c'}*305.2 + $count{'g'}*345.2 + $count{'n'} * 320.5 + 159;
+	}elsif($type eq "dsdna"){
+	    return length($seq) * 607.4 + 157.9;
+	}else{
+	    die("Unknown type $type in molecular_weight()");
+	}
+    }
 }
 
 
@@ -416,7 +455,7 @@ sub view_cds{
 	
 	for ($i = 0; $i < $length * 4 + 6 + $gap; $i ++){
 	    printf OUT "%d,%3.2f,%3.2f,%3.2f,%3.2f\n", $i + 1, 
-	    $a[$i], $t[$i], $g[$i], $c[$i];
+	    $data->{a}[$i], $data->{t}[$i], $data->{g}[$i], $data->{c}[$i];
 	}
 	close(OUT);
     }
@@ -479,4 +518,73 @@ sub filter_cds_by_atg{
 }
 
 
+
+=head2 refseq2gbk
+
+  Name: refseq2gbk   -   convert RefSeq and GenBank IDs of prokaryotes 
+
+  Description:
+    This method converts the RefSeq and GenBank IDs of prokaryotic genomes
+    based on the list provided by NCBI at
+    ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/prokaryotes.txt
+    Version suffix is ignores. For example, NC_000913.1 is converted to U00096.
+
+ Usage: 
+    $newid = refseq2gbk($id);
+
+ Options:
+    None.
+
+  Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20140512-01 initial posting.
+
+=cut
+
+
+
+sub refseq2gbk{
+    my $id = shift;
+
+    $id =~ s/\.\d+//g;
+
+    require LWP::Simple;
+
+    my $url = 'ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/prokaryotes.txt';
+    my $dir = _sdb_path() . '/data/prokaryote.txt';
+    my $rc = LWP::Simple::mirror($url, $dir);
+    die("refseq2gbk: cannot retrieve data from NCBI.") unless(-e $dir);
+
+    my %refseq2gbk;
+    for my $line (readFile($dir)){
+	my @F = split(/\t/, $line);
+
+	$F[8] =~ s/-//g;
+	$F[9] =~ s/-//g;
+
+	$F[8] =~ s/\.\d+//g;
+	$F[9] =~ s/\.\d+//g;
+
+	if($F[8] =~ /,/ || $F[9] =~ /,/){
+	    
+	    my $i = 0;
+	    my @right = split(/,/, $F[9]);
+	    for my $tmp (split(/,/, $F[8])){
+		$refseq2gbk{$tmp} = $right[$i];
+		$refseq2gbk{$right[$i]} = $tmp;
+		$i ++;
+	    }
+	}else{
+	    $refseq2gbk{$F[8]} = $F[9];
+	    $refseq2gbk{$F[9]} = $F[8];
+	}
+    }
+
+#    p %refseq2gbk;
+
+    return $refseq2gbk{$id};
+}
+
+
 1;
