@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -32,6 +32,7 @@ use base qw(Exporter);
 use XML::Simple;
 
 use G::Shell::Help;
+use G::Messenger;
 
 our @EXPORT = qw(
 		 pubmed
@@ -179,6 +180,11 @@ sub entrez {
 	return;
     }
 
+    if($db eq 'genome'){
+	$keyword .= '[Organism] AND "complete genome"[title] AND "chromosome"[title]';
+	$db = 'nucleotide';
+    }
+
     my $ua = new LWP::UserAgent;
 
     my %params = (
