@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -248,9 +248,9 @@ sub generateGMap{
   <meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
     <title>Google Map View of $name - generated by G-language GAE</title>
 
-    <script src="http://maps.google.com/maps?file=api&amp;v=2.156&amp;key=$apikey"
+    <script src="http://maps.google.com/maps?file=api&v=2&key=$apikey"
             type="text/javascript" charset="utf-8"></script>
-    <script src="http://googlemaps.googlermania.com/gmapkit/api?js=gmapkitmapcontrols3d.js" type="text/javascript"></script>
+<!--    <script src="http://googlemaps.googlermania.com/gmapkit/api?js=gmapkitmapcontrols3d.js" type="text/javascript"></script> -->
 
     <script type="text/javascript">
     //<![CDATA[
@@ -349,7 +349,8 @@ sub generateGMap{
 
         //Now create the custom map. Would normally be G_NORMAL_MAP,G_SATELLITE_MAP,G_HYBRID_MAP
         map = new GMap2(document.getElementById("map"), {mapTypes:[pic_customMap]});
-        map.addControl(new GMapKitMapControls3d()); //GLargeMapControl());
+        //map.addControl(new GMapKitMapControls3d());
+        map.addControl(new GLargeMapControl());
         map.addControl(new GMapTypeControl());
 	map.addControl(new GOverviewMapControl());
         map.enableDoubleClickZoom();
@@ -357,6 +358,28 @@ sub generateGMap{
 	map.enableScrollWheelZoom();
         map.setCenter(new GLatLng(0, 0), 2, pic_customMap);
 
+///////// Definition of Zoom range v
+        var minZoom= 1;
+	var maxZoom= $maxlevel;
+	var spoint = map.getCenter();
+	var szoom = map.getZoom();
+	if (szoom > maxZoom) {
+	       szoom = maxZoom;
+	} else if (szoom < minZoom) {
+	       szoom = minZoom;
+	}
+	var maptypes = map.getMapTypes();
+	for (var i = 0; i < maptypes.length; i++) {
+	     maptypes[i].getMinimumResolution = function() {
+	          return minZoom;
+             };
+             maptypes[i].getMaximumResolution = function() {
+                  return maxZoom;
+             };
+        }
+///////// Definition of Zoom range ^
+
+
 /////////////////////////////////////////////////////////////////////////////////////
 //Add any markers here e.g.
 //      map.addOverlay(new GMarker(new GLatLng(x,y)));
