@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -29,10 +29,17 @@ package Rcmd::Clustering;
 
 use strict;
 use autouse 'Carp'=>qw(croak);
+use base qw(Exporter);
 
 use SubOpt;
 use G::Messenger;
 
+our @EXPORT = qw(
+		 hclust
+		 som
+		 kmeans
+		 );
+
 #:::::::::::::::::::::::::::::::::
 #       Perldoc
 #:::::::::::::::::::::::::::::::::
@@ -88,24 +95,21 @@ sub sample_data_for_clustering{
 }
 
 
-=head2 $rcmd->som()
+=head2 som()
 
-  Name: $rcmd->som()   -   clustering using Self-Organizing Map
+  Name: som()   -   clustering using Self-Organizing Map
 
   Description:
     Clustering with Self-Organizing Map (SOM) using R language.
     Installation of GeneSOM library for R language is required.
         run R as a super user - sudo R - and type the following:
-        install.packages('GeneSOM')) 
+        install.packages('som')) 
     
     Returns a two-dimensional array correspondingn to the 
     result$visual of som() in R's GeneSOM library.
 
   Usage:
-    $rcmd = new Rcmd();
-    @result = $rcmd->som(\@array1, \@array2, \@array3, ..., -label=>\@label);
-      or
-    @result = $rcmd->som(-sampledata=>1);
+    @result = som(\@array1, \@array2, \@array3, ..., -label=>\@label);
 
     Arrays correspond to the columns (data series), and labels for each of
     these arrays can be given by -label option.
@@ -114,7 +118,7 @@ sub sample_data_for_clustering{
    -label        labels or names of the data series.
    -xdim         x-dimension of the map (default: 3)
    -ydim         y-dimension of the map (default: 3)
-   -filename     output filename of the graph (default: som.png)
+   -filename     output filename of the graph (default: som.pdf)
    -output       output toggle option (default: show)
                  "g" to generate graph without displaying.
    -sampledata   use sample data (default: 0)
@@ -123,6 +127,7 @@ sub sample_data_for_clustering{
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+   20141110-01 exported by default
    20070612-01 converted to Rcmd::Clustering
    20030904-01 initial posting (G::Tools::RCluster)
 
@@ -130,10 +135,10 @@ sub sample_data_for_clustering{
 
 
 sub som{
-    my $rcmd = shift;
+    my $rcmd = new Rcmd();
     $rcmd->set_mode('tmp');
 
-    opt_default(filename=>"som.png", output=>"show", xdim=>3, ydim=>3, sampledata=>0, topo=>'hexa', neigh=>'gaussian');
+    opt_default(filename=>"som.pdf", output=>"show", xdim=>3, ydim=>3, sampledata=>0, topo=>'hexa', neigh=>'gaussian');
     my @args       = opt_get(@_);
     my $xdim       = opt_val("xdim");
     my $ydim       = opt_val("ydim");
@@ -150,16 +155,8 @@ sub som{
 	$rcmd->set_clust_data(@args, -label=>$label);
     }
 
-    my $message = $rcmd->exec(
-			      'require(som)'
-			      );
-
-    croak(
-	"\n\nPackage som not installed.\nrun\n\tinstall.packages(\'som\')\n" . 
-	"in R as root.\n\n"
-	) if ($message =~ /FALSE/);
-
     $rcmd->exec(
+		'require(som)',
 		"rclust.som<-som(rclust, $xdim, $ydim, topo='hexa', neigh='gaussian')",
 		'rclust.som$visual'
 		);
@@ -182,12 +179,10 @@ sub som{
 
     if($output =~ /g/ || $output =~ /show/){
 	$rcmd->exec(
-		    'postscript("/tmp/som.ps")',
+                    "pdf('./graph/".$filename."')",
 		    'plot(rclust.som)'
 		    );
 
-	mkdir('graph', 0777);
-	system("convert -rotate 90 /tmp/som.ps graph/$filename");
 	msg_gimv("graph/$filename") if ($output =~ /show/);
     }
     
@@ -197,101 +192,118 @@ sub som{
 
 
 
-=head2 $rcmd->hclust()
 
-  Name: $rcmd->hclust()   -   hierarchical clustering analysis
 
-  Description:
-    Hierarchical clustering using R language.
-    Distance measure to use can be specified by -dist_method option.
-    (euclidean:default, maximum, manhattan, canberra, binary)
-    Clustering method can be specified by -hclust_method option. 
-    (complete:default, ward, single, average, mcquitty, median, centroid)
 
-    This method automatically runs dist() in R to calculate the dissimilarities
-    of the given data and passes the result to hclust() of R.
+=head2 hclust
+
+  Name: hclust   -   Hierarchical clustering analysis for given arraies
+
+  Descriptions:
+    Hierarchical clustering analysis methods for given arrays.
+
+    Ward method uses the ward.D2 by default.
+
+    Installation of amap library for R language is required.
+      run R as a super user - sudo R - and type the followings:
+        install.packages('amap')
 
-    Returned value corresponds to result$merge of hclust() in R (n-1 by 2 matrix).
-    Row i describes the merging of clusters at step i.
-    
   Usage:
-    $rcmd = new Rcmd();
-    @merge = $rcmd->hclust(\@array1, \@array2, \@array3, ..., -label=>\@label);
+    hclust(\@array1_of_values, \@array2_of_values, ...);
       or
-    @merge = $rcmd->hclust(-sampledata=>1);
-
-    Arrays correspond to the columns (data series), and labels for each of
-    these arrays can be given by -label option.
+    hclust(\@array1_of_values, \@array2_of_values, ..., -label => \@grouping_label);
 
  Options:
-   -label           labels or names of the data series.
-   -dist_method     distance measure (euclidean:default, maximum, manhattan, 
-                    canberra, binary)
-   -hclust_method   clustering method (complete:default, ward, single, average, 
-                    mcquitty, median, centroid) 
-   -filename        output filename of the graph (default: hclust.png)
-   -output          output toggle option (default: show)
-                    "g" to generate graph without displaying.
-   -sampledata      use sample data (default: 0)
+   -output       output toggle option (default: show)
+                 "g" to generate graph without displaying.
+   -filename     output filename of the clustering graph (default: hclust.pdf)
+   -method       the agglomeration method to be used (default: ward).
+                   'ward', 'single', 'complate', 'average', 'centroid', 'median' or 'mcquitty'
+   -distmethod   the distance measure method (default: correlation)
+                   'euclidean', 'maximum', 'manhattan', 'canberra', 'binary', 'kendall'
+                   'spearman', 'pearson' (not centered Pearson), 'abspearson' (Absolute Pearson),
+                   'correlation' (Centered Pearson) or 'abscorrelation' (Absolute correlation)
+                 this option is based on 'Dist' method in 'amap' library in R.
+   -label        labels or names of the data series
 
-  Author: 
-    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+  Author:
+     Kazuki Oshita (cory@g-language.org)
 
   History:
-   20070612-01 converted to Rcmd::Clustering
-   20030904-01 initial posting (G::Tools::RCluster)
+     20130321-01 complete rewrite by cory, exported by default
+     20070612-01 converted to Rcmd::Clustering
+     20030904-01 initial posting (G::Tools::RCluster)
 
 =cut
 
-sub hclust{
-    my $rcmd = shift;
-    $rcmd->set_mode('tmp');
 
-    opt_default("dist_method"=>"euclidean", "hclust_method"=>"com", 
-		filename=>"hclust.png", output=>"show", sampledata=>0);
-    my @args = opt_get(@_);
-    my $dist_method = opt_val("dist_method");
-    my $hclust_method = opt_val("hclust_method");
-    my $filename= opt_val("filename");
-    my $output = opt_val("output");
-    my $label = opt_val("label");
-    my $sampledata = opt_val("sampledata");
+sub hclust {
+    &opt_default(output => 'show', label => [], filename => 'hclust.pdf', method => 'ward', distmethod => 'correlation');
 
-    if($sampledata){
-	$rcmd->sample_data_for_clustering();
-    }else{
-	$rcmd->set_clust_data(@args, -label=>$label);
+    my @args= opt_get(@_);
+    my $output= opt_val('output');
+    my @label=  @{opt_val('label')};
+    my $filename= opt_val('filename');
+    my $method= opt_val('method');
+    my $distmethod= opt_val('distmethod');
+
+    my @all_methods= ('ward', 'single', 'complate', 'average', 'centroid', 'median', 'mcquitty');
+    unless (grep /^$method$/, @all_methods) {
+        $method= 'ward';
+    }
+    $method = 'ward.D2' if ($method = 'word');
+
+    my @all_dist_methods= ('euclidean', 'maximum', 'manhattan', 'canberra', 'binary', 'pearson',
+                           'abspearson', 'correlation', 'abscorrelation', 'spearman', 'kendall');
+    unless (grep /^$distmethod$/, @all_dist_methods) {
+        $distmethod= 'pearson';
+    }
+
+    my $rcmd = Rcmd->new();
+
+    my (%data_table, @R_names);
+    for my $i (0 .. $#args) {
+        my $R_name= 'array'.$i;
+        push @R_names, $R_name;
+
+        $data_table{$R_name}= $_[$i];
+        $rcmd->array($R_name, @{$args[$i]});
+    }
+
+    # 'label' list object (if required)
+    $rcmd->sarray('label', @label) if $#label > -1;
+    my @R_commands= (
+                     'CMP <- complete.cases('.join(', ', @R_names).')',
+                     ('d.table <- data.frame('.$R_names[0].'=1:'.($#{$data_table{$R_names[0]}}+1).', row.names=label)')x!! ($#label > -1),
+                     ('d.table <- data.frame('.$R_names[0].'=1:'.($#{$data_table{$R_names[0]}}+1).')')x!!                  ($#label == -1)
+		     );
+
+    for my $key (@R_names) {
+        push @R_commands, $key.' <- '.$key.'[CMP]';
+        push @R_commands, 'd.table$'.$key.' <- '.$key;
     }
 
     $rcmd->exec(
-		'require(stats)',
-		"rclust.hclust<-hclust(dist(rclust, \"$dist_method\"), \"$hclust_method\")"
-		);
+                @R_commands,
 
-    my @result = $rcmd->exec('rclust.hclust$merge');
+                'library("amap")',
+                "pdf('./graph/".$filename."')",
+                'hc <- hclust(Dist(d.table, method="'.$distmethod.'"), method="'.$method.'")',
+                'plot(hc)',
+               );
 
-    if($output =~ /g/ || $output =~ /show/){
-	$rcmd->exec(
-		    'postscript("/tmp/rclust.ps")',
-		    'plot(rclust.hclust,hang=-1)'
-		    );
+    msg_gimv('graph/'.$filename) if $output eq 'show';
 
-	mkdir('graph', 0777);
-	system("convert -rotate 90 /tmp/rclust.ps graph/$filename");
-	msg_gimv("graph/$filename") if ($output =~ /show/);
-    }
-
-    shift @result;
-
-    $rcmd->set_mode();
-    return @result;
+    return '';
 }
 
 
 
-=head2 $rcmd->kmeans()
 
-  Name: $rcmd->kmeans()   -   clustering with K-means method
+
+=head2 kmeans()
+
+  Name: kmeans()   -   clustering with K-means method
 
   Description:
     
@@ -303,19 +315,16 @@ sub hclust{
     (a vector of cluster numbers to which each point is allocated)
     
   Usage:
-    $rcmd = new Rcmd();
     @cluster = $rcmd->kmeans(\@array1, \@array2, \@array3, ..., -label=>\@label);
-      or
-    @cluster = $rcmd->kmeans(-sampledata=>1);
 
     Arrays correspond to the columns (data series), and labels for each of
     these arrays can be given by -label option.
 
   Options:
    -label           labels or names of the data series.
-   -center          number of cluster centers (default: 5)
+   -centers          number of cluster centers (default: 5)
    -iter.max        number of iterations (default: 10)
-   -filename        output filename of the graph (default: hclust.png)
+   -filename        output filename of the graph (default: kmeans.pdf)
    -output          output toggle option (default: show)
                     "g" to generate graph without displaying.
    -sampledata      use sample data (default: 0)
@@ -331,10 +340,10 @@ sub hclust{
 
 
 sub kmeans{
-    my $rcmd = shift;
+    my $rcmd = new Rcmd();
     $rcmd->set_mode('tmp');
 
-    opt_default("iter.max"=>10, filename=>"kmeans.png", output=>"show", sampledata=>0, centers=>5);
+    opt_default("iter.max"=>10, filename=>"kmeans.pdf", output=>"show", sampledata=>0, centers=>5);
     my @args = opt_get(@_);
     my $centers = opt_val("centers");
     my $iter = opt_val("iter.max");
@@ -357,13 +366,11 @@ sub kmeans{
 
     if($output =~ /g/ || $output =~ /show/){
 	$rcmd->exec(
-		    'postscript("/tmp/kmeans.ps")',
+		    "pdf('./graph/".$filename."')",
 		    'plot(rclust,col=rclust.kmeans$cluster)',
 		    "points(rclust.kmeans\$centers, col=1:$centers,pch=8)"
 		    );
 
-	mkdir('graph', 0777);
-	system("convert -rotate 90 /tmp/kmeans.ps graph/$filename");
 	msg_gimv("graph/$filename") if ($output =~ /show/);
     }
     
