@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2011 Keio University
+#     Copyright (C) 2001-2014 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: SubOpt.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -43,7 +43,7 @@ our @EXPORT = qw(
 		 opt_as_gb
 );
 
-our $VERSION = '0.61';
+our $VERSION = '0.62';
 
 #::::::::::::::::::::::::::::::
 #          Variables
@@ -79,9 +79,7 @@ sub opt_as_gb {
     my $ref = \$unknown;
     my $ref2 = ref $unknown;
 
-    if ($ref2 =~ /Bio::Seq/){
-	return new G::IO($unknown, 'bioperl', 'no msg');
-    }elsif ("$unknown" =~ /\=/){
+    if ("$unknown" =~ /\=/){
 	if (defined $unknown->{SEQ}){
 	    return $unknown;
 	}else{
@@ -146,11 +144,8 @@ sub opt_get{
 	    }
         }else{
 	    my $ref = ref $args[$i];
-	    if ($ref =~ /Bio::Seq/){
-		push(@new_args, new G::IO($args[$i], "bioperl", "no msg"));
-	    }else{
-		push(@new_args, $args[$i]);
-	    }
+	    push(@new_args, $args[$i]);
+
 	    $i ++;
 	}
     }
